unit fMHA_ASI_ProgressNote;

interface

uses
  Windows,
  Messages,
  SysUtils,
  System.Contnrs,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  ExtCtrls,
  ORCtrls,
  StdCtrls;

type
  TfrmMHA_ASI_ProgressNote = class(TForm)
    pnlBottom: TPanel;
    btnOK: TJvXPButton;
    btnCancel: TJvXPButton;
    cbxStaff: TORComboBox;
    lblStaff: TLabel;
    procedure FormCreate(Sender: TObject);

    procedure ConfigureForm;
    procedure cbxStaffChange(Sender: TObject);
    procedure cbxStaffNeedData(Sender: TObject; const StartFrom: string;
      Direction, InsertAt: Integer);
    procedure cbxStaffDropDown(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMHA_ASI_ProgressNote     : TfrmMHA_ASI_ProgressNote;

implementation

{$R *.dfm}

uses
  uIA_GUIControl,
  fIA_dialogs,
  uMHA_Const,
  uIA_Const,
  uIA_Utils,
  uIA_Staff_OPS,
  uMHA_ASI,
  uMHA_ASI_OPS,
  uIA_ThisUser,
  uMHA_DM;

procedure TfrmMHA_ASI_ProgressNote.FormCreate(Sender: TObject);
begin
  ConfigureForm;
end;


procedure TfrmMHA_ASI_ProgressNote.ConfigureForm;
begin
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;
      Position := poMainFormCenter;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      pnlBottom.Height := ButtonHeight * 2;
      Height := ButtonHeight * 12;

      Width := 2 * Height;

      Left := GetLeft(Width);
      Top := GetTop(Height);

      lblStaff.Top := ButtonHeight;
      cbxStaff.Top := lblStaff.Top + 3 * lblStaff.Height div 2;
      btnOK.Top := pnlBottom.Height div 4;
      btnCancel.top := btnOk.Top;
      btnOk.Height := ButtonHeight;
      btnCancel.Height := btnOk.Height;
      btnCancel.width := ButtonWidthNormal;
      btnOK.width := ButtonWidthNormal;
      btnOk.Left := pnlBottom.width - (3 * btnOk.Width div 2);
      btnCancel.Left := btnCancel.Width div 2;

      cbxStaff.Left := btnCancel.Left;
      cbxStaff.Height := 3 * TextHeight('Yy', gvFont) div 2;
      cbxStaff.Width := Width - 2 * btnCancel.Left;

      lblStaff.Left := btnCancel.Left;
      lblStaff.Height := 2 * cbxStaff.Height;
      lblStaff.Width := cbxStaff.Width;

      {if gvUseScreenReader then
        begin
          btnCancel.Caption := btnCancel.Caption + ' button';
          btnOK.Caption := btnOK.Caption + ' button';
        end;
       }
      ActiveControl := cbxStaff;

    end;
end;

procedure TfrmMHA_ASI_ProgressNote.cbxStaffNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;

  (Sender as TORComboBox).ForDataUse(Staff_OPS.LoadStaffSubSet(StartFrom,
    Direction, DataModule1.RPCBroker1));
end;

procedure TfrmMHA_ASI_ProgressNote.cbxStaffDropDown(Sender: TObject);
begin
 // OnFormShow does not load cbxStaff items because the method is very
 // slow. This method will load the items the first time the user clicks
 // on the combobox's down arrow (i.e., when item.count = 0

  if (cbxStaff.Items.Count = 0) then
    begin

      if gvCoSignerIEN = '' then
        begin
          cbxStaff.InitLongList(ThisUser.Name);
          // Patch 129 fix for DUZ's with greater than 9 digits ; changed StrtoInt to StrToInt64 llh
          cbxStaff.SelectByIEN(StrToInt64(ThisUser.DUZ));
        end
      else
        begin
          cbxStaff.InitLongList(Piece(gvCoSignerIEN, U, 1));
          // Patch 129 fix for DUZ's with greater than 9 digits ; changed StrtoInt to StrToInt64 llh
          cbxStaff.SelectByIEN(StrToInt64(Piece(gvCoSignerIEN, U, 2)));
        end;
    end;

end;

procedure TfrmMHA_ASI_ProgressNote.cbxStaffChange(Sender: TObject);
var
  tmpDUZ                      : string;
begin
  tmpDUZ := IntToStr(cbxStaff.ItemIEN);

  if (tmpDUZ = '0') or (tmpDUZ = '') or (tmpDUZ = ' ') then
    begin
      gvCoSignerIEN := '';
      exit;
    end
  else
    gvCoSignerIEN := cbxStaff.text + U + tmpDUZ;
end;

procedure TfrmMHA_ASI_ProgressNote.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_ASI_ProgressNote.btnOKClick(Sender: TObject);
begin

  if UpperCase(cbxStaff.Text) = UpperCase(ThisUser.Name) then
    begin
      MsgDlgFrm('You must select a co-signer (it can not be yourself).',
        MHA_CAPTION, mtInformation, [mbOK], 0);
      exit;
    end;

  if cbxStaff.Text = '' then
    begin
      MsgDlgFrm('You must select a co-signer.',
        MHA_CAPTION, mtInformation, [mbOK], 0);
      exit;
    end;

  if not ASI_OPS.SaveProgressNote(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS) then
    MsgDlgFrm('The ASI progress note was not written to VistA database.',
      MHA_Caption, mtError, [mbOK], 0);

  Close;
end;

end.

