unit fMHA_InstrumentDescription;

interface

uses
  Classes,
  Windows,
  Messages,
  SysUtils,
  System.UITypes,
  Controls,
  Forms,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  Menus,
  RzLabel,
  RzPanel,
  uMHA_Administration,
  uIA_Instrument,
  Printers,
  Dialogs,
  Vcl.Graphics;

type

  TfrmMHA_InstrumentDescription = class(TForm)
    memDescription: TRichEdit;
    dlgSave: TSaveDialog;
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    pnlTop: TPanel;
    pnlHeader: TRzPanel;
    lblHeader: TRzLabel;
    mnuSaveAs: TMenuItem;
    mnuPrint: TMenuItem;
    mnuHelp: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    N2: TMenuItem;
    mnuAbout: TMenuItem;
    Edit1: TMenuItem;
    mnuCopy: TMenuItem;
    imgBackup: TImage;
    mnuPrintBlank: TMenuItem;
    PrintDialog: TPrintDialog;
    procedure FormShow(Sender: TObject);
    procedure ConfigureForm;

    procedure mnuSaveClick(Sender: TObject);
    procedure mnuCopyClick(Sender: TObject);
    procedure mnuPrintClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure mnuOnlineSupportClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure imgBackupClick(Sender: TObject);
    procedure mnuPrintBlankClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowDescription(aInstrument: TInstrument);
    procedure ShowIncompleteAdministration(Admin: TAdministration);
  end;

var
  frmMHA_InstrumentDescription: TfrmMHA_InstrumentDescription;

implementation

uses
  uIA_GUIControl,
  uMHA_Registry,
  uIA_Utils,
  uIA_Const,
  uMHA_Const,
  fMHA_About,
  fMHA_Print_Blank;

const
  clIndent                    = '   ';


{$R *.DFM}

var
  lvInitialize                : Boolean;


procedure TfrmMHA_InstrumentDescription.FormCreate(Sender: TObject);
begin
  lvInitialize := True;
  Position := poDesigned;
  ConfigureForm;
  memDescription.Clear;
end;


procedure TfrmMHA_InstrumentDescription.FormShow(Sender: TObject);
begin
// if not Administration.IsComplete and not (Administration.RecordID = NO_ID)
//    then
//    ShowIncompleteAdministration;

  lvInitialize := False;
  Screen.Cursor := crDefault;
  ActiveControl := memDescription;
end;

procedure TfrmMHA_InstrumentDescription.ConfigureForm;
begin
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Height := GetHeight(gvHeight - 30); //CheckHeight(400, Height);
      Width := GetWidth(gvWidth - 30); //CheckWidth(400, Width);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlHeader.Height := pnlHeaderHeight;
      with lblHeader do
        begin
          ShowHint := True;
          Font.Assign(gvFont);
          Font.Size := lblHeaderFontSize;
          Font.Color := clBlack;
          Height := lblHeaderHeight;
          AutoSize := True;
          Layout := tlCenter;
          Align := alClient;
          Color := clWhite;
          //Width := 4 * pnlHeader.Width div 5;
        end;

      if gvUseScreenReader then
        begin
          Left := 0;
          Width := Screen.Width;
        end;
    end;
end;

procedure TfrmMHA_InstrumentDescription.ShowIncompleteAdministration(Admin: TAdministration);
begin

  with memDescription, Admin do
    begin
      if gvUseScreenReader then
        Caption := 'Incomplete administration of ' + InstrumentName +
          ' Press escape key to Exit.';

      lblHeader.Caption := '   Incomplete Administration of ' + InstrumentName;
      Lines.Add('');
      Lines.Add(clIndent + 'Date/Time Begun: ' + DateTimeToStr(DateGiven));
      Lines.Add(clIndent + 'Date/Time Last Saved: ' + DateTimeToStr(DateSaved));
      Lines.Add(clIndent + 'Number of Questions Answered: ' +
        IntToStr(NumberAnswered));
      Lines.Add(clIndent);
      Font.Name := 'Courier New';
      Font.Size := gvFont.Size;
      Font.Color := gvFont.color;
      Font.Style := gvFont.style;
      MaxLength := Length(Text);
    end;
end;

procedure TfrmMHA_InstrumentDescription.ShowDescription(aInstrument: TInstrument);
begin
  with memDescription, aInstrument do
    begin
      gvNameOfInstrument := InstrumentName; // used for printing
      if gvUseScreenReader then
        Caption := 'Description of ' + InstrumentName +
          ' Press escape key to Exit.';

      lblHeader.Caption := '   Description of ' + InstrumentName;
      lblHeader.Hint := PrintTitle;

      Lines.Add('');
      Lines.Add(clIndent + 'Clinical Features');
      Lines.Add('');
      Lines.Add(clIndent + clIndent + 'Print Title:  ' + PrintTitle);
      Lines.Add(clIndent + clIndent + 'Version:  ' + Version);
      Lines.Add(clIndent + clIndent + 'Author:  ' + Author);
      Lines.Add(clIndent + clIndent + 'Publisher:  ' + Publisher);
      Lines.Add(clIndent + clIndent + 'Copyright:  ' + Copyright);
      Lines.Add(clIndent + clIndent + 'Publication Date:  ' + PublicationDate);
      Lines.Add(clIndent + clIndent + 'Reference:  ' + Reference);
      Lines.Add(clIndent + clIndent + 'Purpose:  ' + Purpose);
      Lines.Add(clIndent + clIndent + 'Norm Sample:  ' + NormSample);
      Lines.Add(clIndent + clIndent + 'Target Population:  ' + TargetPopulation);
      Lines.Add('');
      Lines.Add('');
      Lines.Add(clIndent + 'Technical Features');
      Lines.Add('');
      Lines.Add(clIndent + clIndent + 'Administration Privilege:  ' + A_PRIVILEGE);
      Lines.Add(clIndent + clIndent + 'Results Privilege:  ' + R_PRIVILEGE);
      Lines.Add(clIndent + clIndent + 'Entered By:  ' + EnteredBy);
      Lines.Add(clIndent + clIndent + 'Entry Date:  ' + EntryDate);
      Lines.Add(clIndent + clIndent + 'Last Edited By:  ' + LastEditedBy);
      Lines.Add(clIndent + clIndent + 'Last Edit Date:  ' + LastEditDate);
      Lines.Add(clIndent + clIndent + 'National Test:  ' + BoolToStr(IsNationalTest));
      Lines.Add(clIndent + clIndent + 'Requires License:  ' +
        BoolToStr(RequiresLicense));
      if RequiresLicense then
        Lines.Add(clIndent + clIndent + 'License is Current:  '
          + BoolToStr(LicenseIsCurrent));
      Lines.Add(clIndent + clIndent + 'Is Legacy Instrument:  ' +
        BoolToStr(IsLegacy));
      Lines.Add(clIndent + clIndent + 'Submit To National DB:  ' +
        BoolToStr(SubmitToNationalDB));
      Lines.Add(clIndent);
      Lines.Add(clIndent);

      Font.Name := 'Courier New';
      Font.Size := gvFont.Size;
      Font.Color := gvFont.color;
      Font.Style := gvFont.style;
      MaxLength := Length(Text);
    end; //with memDescription
end;

procedure TfrmMHA_InstrumentDescription.mnuSaveClick(Sender: TObject);
begin
  with dlgSave do
    begin
      FileName := 'Instrument Descriptions.rtf';
      Filter := 'Rich text file (*rtf)|*.rtf';
      if Execute then
        memDescription.Lines.SaveToFile(FileName);
    end;
end;

procedure TfrmMHA_InstrumentDescription.mnuCopyClick(Sender: TObject);
begin
  with memDescription do
    begin
      SelectAll;
      CopyToClipboard;
    end;
end;

procedure TfrmMHA_InstrumentDescription.mnuPrintClick(Sender: TObject);
var
  i,
    lvLeftMargin,
    lvLineHeight,
    lvLinesPerPage,
    lvAmountPrinted           : Integer;

begin

  if not PrintDialog.Execute then
    Exit;

  with Printer do
    begin
      Canvas.Font.Name := 'Courier New';
      Canvas.Font.Style := [];
      Canvas.Font.Pitch := fpFixed;
      Canvas.Font.Size := 10;
      lvLinesPerPage := 80;
      lvLineHeight := PageHeight div lvLinesPerPage;
      lvAmountPrinted := 3 * lvLineHeight;

      lvLeftMargin := GetDeviceCaps(Printer.Handle, LOGPIXELSX) div 2;

      BeginDoc;

      for i := 0 to memDescription.lines.Count - 1 do
        begin
          Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, memDescription.lines[i]);
          lvAmountPrinted := lvAmountPrinted + lvLineHeight;
        end;

      EndDoc;
    end;
end;

procedure TfrmMHA_InstrumentDescription.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_InstrumentDescription.mnuOnlineSupportClick(
  Sender: TObject);
begin
  VisitURL(MHA_ONLINE_SUPPORT_URL);
end;

procedure TfrmMHA_InstrumentDescription.mnuAboutClick(Sender: TObject);
begin

  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try
end;

procedure TfrmMHA_InstrumentDescription.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;

procedure TfrmMHA_InstrumentDescription.imgBackupClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_InstrumentDescription.mnuPrintBlankClick(
  Sender: TObject);
begin

  try
    frmPrint_Instrument := TfrmPrint_Instrument.Create(nil);
  finally
    frmPrint_Instrument.Free;
  end; //try

end;

procedure TfrmMHA_InstrumentDescription.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

end.

