unit uMHA_ASI_OPS;

interface

uses
  Classes,
  SysUtils,
  Contnrs,
  dialogs,
  uMHA_ASI;

type
  TASI_OPS = class
  private
    FLastASIDate: string;
    FSignedInterviewsExist: Boolean;
    FChoicesMade,
      FItemValues: TStringList;
    FListOfASIResults: TObjectList;
  protected
  public
    destructor Destroy; override;
    constructor Create;

    property LastASIDate: string read FLastASIDate write FLastASIDate;
    property ListOfASIResults: TObjectList read FListOfASIResults write FListOfASIResults;
    property SignedInterviewsExist: BOOLEAN read FSignedInterviewsExist write FSignedInterviewsExist;
    property ChoicesMade: TStringList read FChoicesMade write FChoicesMade;
    property ItemValues: TStringList read FItemValues write FItemValues;

    function Sign(ASignature, aIENS: string): Boolean;
    function SaveProgressNote(aIENS: string): Boolean;

    function SaveNewInterview(aPatientDFN,
      aDate,
      aInterviewerDUZ,
      aOrderedbyDUZ: string;
      aInterviewType: TInterviewType): string;

    function SaveChoicesMade(aIENS_Exist: boolean;
      aIENS,
      aPatientDFN,
      aDate,
      aInterviewerDUZ,
      aOrderedbyDUZ: string;
      aInterviewType: TInterviewType): string;


    procedure LoadNew(AInterviewType: TInterviewType);
    function LoadPreviousASIs(aDFN: string): Boolean;
    function LoadReport(aIENS: string; aReportType: TReportType): string;
    procedure LoadTrends(aIENS: string);

    function LoadEvaluationFactors(aIENS: string): string;
    function LoadSeverityRatings(aIENS: string): string;
    function Synch(aIENS: string): Integer;
    function StrToInterviewType(TypeStr: string): TInterviewType;
    function InterviewTypeToStr(aType: TInterviewType): string;

  end; { TASI_List }

var
  ASI_OPS                     : TASI_OPS;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uIA_Patient,
  uIA_ThisUser,
  uMHA_ASI_IO;


constructor TASI_OPS.Create;
begin
  inherited Create;
  FListOfASIResults := TObjectList.Create;
  FChoicesMade := TStringList.Create;
  FItemValues := TStringList.Create;
end;

destructor TASI_OPS.Destroy;
begin
  if FChoicesMade <> nil then
    FChoicesMade.Free;
  if FItemValues <> nil then
    FItemValues.Free;
  inherited Destroy;
end;

function TASI_OPS.StrToInterviewType(TypeStr: string): TInterviewType;
begin
 //if UpperCase(TypeStr) = 'FULL' then
  Result := itFull; // make default

  if TypeStr = ASI_MV then
    Result := itMV;

  if TypeStr = UpperCase(ASI_LITE) then
    Result := itLite;

  if TypeStr = UpperCase(ASI_FOLLOWUP) then
    Result := itFU;
end;

function TASI_OPS.InterviewTypeToStr(aType: TInterviewType): string;
begin
  Result := ASI_FULL; // make default

  if aType = itMV then
    Result := ASI_MV;

  if aType = itLite then
    Result := ASI_LITE;

  if aType = itFU then
    Result := ASI_FOLLOWUP;
end;


function TASI_OPS.SaveNewInterview(
  aPatientDFN,
  aDate,
  aInterviewerDUZ,
  aOrderedbyDUZ: string;
  aInterviewType: TInterviewType): string;
// result is IENS
var
  TempStr                     : string;

begin
  case aInterviewType of
    itLite: TempStr := '2';
    itFU: TempStr := '3';
    itMV: TempStr := '4';
  else
    TempStr := '1'; //Make Full the default
  end; //case

//  tmpParameters := Patient.DFN + U //1
//    + TempStr + U //2
//    + DateToStr(TASI(FListOfASIResults[gvCurrentResult]).ASIDate) + U //3
//    + TASI(FListOfASIResults[gvCurrentResult]).InterviewerDUZ + U //4
//    + TASI(FListOfASIResults[gvCurrentResult]).OrderedByDUZ; //5

  Result := ASIWriteNewInterview(
    aPatientDFN,
    TempStr,
    aDate,
    aInterviewerDUZ,
    aOrderedbyDUZ); // uMHA_ASI_IO
  // Returns IENS, or BAD_DATA.
end;

function TASI_OPS.SaveChoicesMade(aIENS_Exist: boolean;
  aIENS,
  aPatientDFN,
  aDate,
  aInterviewerDUZ,
  aOrderedbyDUZ: string;
  aInterviewType: TInterviewType): string;
var
  TempIENS                    : string;
begin
  TempIENS := '00';
  Result := BAD_DATA;

  //if not TASI(FListOfASIResults[gvCurrentResult]).IENSExist then
  //Result := SaveNewInterview(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType);
  if not aIENS_Exist then
    TempIENS := SaveNewInterview(
      aPatientDFN,
      aDate,
      aInterviewerDUZ,
      aOrderedbyDUZ,
      aInterviewType);

  if TempIENS = BAD_DATA then
      exit;

  if TempIENS = '00' then
    TempIENS := aIENS;

  Result := ASIWriteChoicesMade(TempIENS); // uMHA_ASI_IO
   // When successful this is returned:
   // "OK ASI SAVED XXXX," where XXXX is the RecordIEN
end;

function TASI_OPS.Synch(aIENS: string): Integer;
var
  i                           : Integer;
begin
  Result := 0;
  if FListOfASIResults.Count = 0 then
    Exit;

  for i := 0 to FListOfASIResults.Count - 1 do
    begin
      if TASI(FListOfASIResults[i]).IENS = aIENS then
        begin
          result := i;
          Exit;
        end;
    end;
end;

procedure TASI_OPS.LoadTrends(aIENS: string);
const
  FieldNumbers                = '8.08;9.18;9.19;9.25;10.01;10.04;10.07;10.15;'
    + '10.22;10.25;11.09;11.11;14.31;18.23;18.24;19.23';
begin
  PiecesToList(ASIReadItemValues(aIENS, FieldNumbers),
    U, FItemValues);

  if (FItemValues[0] = BAD_DATA) then
    FItemValues.Clear;
end;

function TASI_OPS.LoadReport(aIENS: string; aReportType: TReportType): string;
var
  AProcedure                  : string;
begin

  if (aReportType = rtNarrative) then
    AProcedure := 'YSRP ASI NARRATIVE'
  else
    AProcedure := 'YSRP ASI ITEM';

  Result := ASIReadReport(aIENS, AProcedure);

end;

function TASI_OPS.LoadEvaluationFactors(aIENS: string): string;
begin
  Result := ASIReadEvaluationFactors(aIENS);
end;

function TASI_OPS.LoadSeverityRatings(aIENS: string): string;
const
  FIELDNUMBERS                = '8.12;9.34;11.18;11.185;14.34;18.29;19.33';
var
  i                           : Integer;
  TempStr, TempStr1           : string;
begin
  Result := '';
  TempStr := ASIReadItemValues(aIENS, FIELDNUMBERS);
  // Format returned is
  // 8.12=12^9.34=^11.18=34^....

  if TempStr = BAD_DATA then
    Exit;

  for i := 0 to NumberofPieces(TempStr, U) do
    begin
      TempStr1 := Piece(Piece(TempStr, U, i), EQ, 2);
      if TempStr1 = '' then
        TempStr1 := 'X';
      Result := Result + TempStr1 + U;
    end;
end;

function TASI_OPS.LoadPreviousASIs(aDFN: string): Boolean;
var
  i                           : Integer;

begin
  Result := False;

  if not ASIReadList(aDFN) then // uMHA_ASI_IO
    Exit;

// G12 does not describe the status of an ASI completely.
// 'INTERVIEW COMPLETED' is used as the first step toward understanding
// its status.

  for i := 0 to ListOfASIResults.Count - 1 do
    begin
      with TASI(ASI_OPS.ListOfASIResults[i]) do
        begin
          if (G12 = 'INTERVIEW COMPLETED') then
            begin
              if IsSigned then
                G12 := 'Completed'
              else
                begin
                  G12 := 'Incomplete';
                  if ASIStatusOfSavedRecord(IENS) = '1^OK TO SIGN' then
                    G12 := 'Ready to sign';
                end;
              if (InterviewTypeStr = ASI_MV) then
                G12 := 'Completed';
            end;
          if (G12 = 'PATIENT REFUSED') then
            G12 := 'Pt Refused';
          if (G12 = 'PATIENT TERMINATED') then
            G12 := 'Pt Terminated';
          if (G12 = 'PATIENT UNABLE TO RESPOND') then
            G12 := 'Unable to respond';

          if UpperCase(OrderedByName) = UpperCase(ThisUser.Name) then
            CanBeSignedByUser := True;
        end;
    end;

  Result := True;
end;

function TASI_OPS.Sign(ASignature, aIENS: string): BOOLEAN;
begin
  Result := ASISign(ASignature, aIENS); // uMHA_ASI_IO
end;

function TASI_OPS.SaveProgressNote(aIENS: string): BOOLEAN;
begin
  Result := ASIWriteProgressNote(aIENS);
end;

procedure TASI_OPS.LoadNew(aInterviewType: TInterviewType);
var
  TempASI                     : TASI;
begin
  TempASI := TASI.Create;

  with TempASI do
    begin
      IENS := NO_ID;
      ASIDate := Now;
      IsSigned := False;
      InterviewerName := ThisUser.Name;
      InterviewerDUZ := ThisUser.DUZ;
      OrderedByName := ThisUser.Name;
      OrderedByDUZ := ThisUser.DUZ;
      CanBeSignedByUser := True;
      IENSExist := False;
      InterviewType := aInterviewType;
      InterviewTypeStr := InterviewTypeToStr(aInterviewType);
    end;

  FListOfASIResults.Add(TempASI);
  gvCurrentResult := FListOfASIResults.Count - 1;

 // TempASI.Free; Note: This will cause a access error.

end;


initialization
  ASI_OPS := TASI_OPS.Create;

finalization
  ASI_OPS.Free;



end.

