unit uMHA_Administration;

interface

uses
  SysUtils,
  uIA_ThisUser,
  Contnrs,
  Classes,
  Controls;

type
  TAdministration = class
    Orderer: TThisUser;
  private
    // Note:  FID is the Administration.ID in the database.
    // FRecordID is an arbitrary number assigned to records in
    // ListOfIncompletes.  It is used by InstrumentSelect
    // to reload an incomplete administration.
    FInstrumentName,
      FID,
      FRecordID,
       FProgNoteTitle,
       FConsultTitle,
      FAdministeredBy,
      FPatientID,
      FOrdererDUZ,
       FConsultIEN,
      FLocationIEN,
      FLocationStr: string;
    FIsRestartable,

      FIsComplete,
      FIsaLegacy,
      FGenerateProgNote,
      FSigned: BOOLEAN;
    FDateGiven,
      FDateSaved: TDateTime;
    FDaysToRestart,
      FNumberAnswered,
      FNumberSkipped,
      FNumberStillRequired: Integer;
    FListOfAdminAnswers: TObjectList;
  protected
  public
    destructor Destroy; override;
    constructor Create;

    property ID: string read FID write FID;
    {** Patient IENS}
    property PatientID: string read FPatientID write FPatientID;
    {** Not required of most instruments}
    property Signed: BOOLEAN read FSigned write FSigned;
    {** DUZ of staff who administered the instrument (this would be the interviewer
    for the ASI or the person who started the pt on the computer for a test}
    property AdministeredBy: string read FAdministeredBy write FAdministeredBy;
    {** Date/time the administration was saved}
    property DateSaved: TDateTime read FDateSaved write FDateSaved;
    {** Date/time the instrument was given to the patient (could be different
    from date saved if given on paper and clerk entered later)}
    property DateGiven: TDateTime read FDateGiven write FDateGiven;
    property InstrumentName: string read FInstrumentName write FInstrumentName;
    {** Last question answered}
    property IsComplete: BOOLEAN read FIsComplete write FIsComplete;
    property GenerateProgNote: BOOLEAN read FGenerateProgNote write FGenerateProgNote;
    //property NumberOfQuestionsAnswered: Integer read FNumberOfQuestionsAnswered
    //  write FNumberOfQuestionsAnswered;
    property IsRestartable: BOOLEAN read FIsRestartable write FIsRestartable;
    property IsaLegacy: BOOLEAN read FIsaLegacy write FIsaLegacy;
    property ConsultIEN: string read FConsultIEN Write FConsultIEN;
    property LocationIEN: string read FLocationIEN write FLocationIEN;
    property LocationStr: string read FLocationStr write FLocationStr;
     //  FIsFirstInstrument;
    property NumberAnswered: Integer read FNumberAnswered write FNumberAnswered;
    property NumberSkipped: Integer read FNumberSkipped write FNumberSkipped;
    property NumberStillRequired: Integer read FNumberStillRequired write FNumberStillRequired;
    property DaysToRestart: Integer read FDaysToRestart write FDaysToRestart;
    property ProgNoteTitle: string read FProgNoteTitle write FProgNoteTitle;
    property ConsultTitle: string read FConsultTitle write FConsultTitle;

    property RecordID: string read FRecordID write FRecordID;
    property OrdererDUZ: string read
      FOrdererDUZ write FOrdererDUZ;
    property ListOfAdminAnswers: TObjectList read FListOfAdminAnswers write FListOfAdminAnswers;
  //published
  end; { TAdministration }


implementation

constructor TAdministration.Create;
begin
  inherited Create;
  FListOfAdminAnswers := TObjectList.Create;
  Orderer := TThisUser.Create;

  //Unless changed, the Orderer and User are the same person
  Orderer.Name := ThisUser.Name;
  Orderer.DUZ := ThisUser.DUZ;
  Orderer.AccessKeys := ThisUser.AccessKeys;

end;


destructor TAdministration.Destroy;
begin
  Orderer.Free;
  ListOfAdminAnswers.Free;
  inherited Destroy;
end;

end.

