unit uMHA_Administration_OPS;

interface

uses
  Contnrs,
  ComCtrls,
  Classes,
  DateUtils,
  SysUtils,
  TRpcb,
  uIA_Instrument,
  uMHA_Administration;

type
  TAdministration_OPS = class
  private
    FAdministrations: TObjectList;
    FListOfIncompletes: TObjectList;
    FProgressNoteStr: TStringList;

    procedure InitializeListOfAnswers(aInstrument: TInstrument;
      aAdmin: TAdministration);

    //function BuildSectionHeader(ACaption: string): string;
    function GetItemIndex(aQuestionID, ChoiceID: string): string;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    procedure ReadIncompleteResults(aDFN, OrdererDUZ: string);
    function ReadCrashFile(aDFN, aDUZ, aInstrName: string): boolean;
    function SaveAdmin(Admin: TAdministration): string;
    function DeleteAdmin(aAdminID: string): string;
    procedure ClearAdmin(aAdmin: TAdministration);
    procedure CopyAdmin(aAdmin, bAdmin: TAdministration);
    procedure LoadAnswers(aInstrument: TInstrument; aAdmin: TAdministration);
    function BuildProgressNote(aAdmin: TAdministration;
      aWriteFullText: Boolean): string;
    procedure SaveProgressNote(aWriteFullText: Boolean;
      aAdmin: TAdministration);
    procedure SaveConsultNote(aWriteFullText: Boolean;
      aAdmin: TAdministration);
    procedure SaveEditedProgressNote(aWriteFullText: Boolean;
      aAdmin: TAdministration);
    procedure SaveEditedConsultNote(aWriteFullText: Boolean;
      aAdmin: TAdministration);
    function SaveScore(aAdmin: TAdministration): string;
    function SaveAnswers(Admin: TAdministration): Boolean;
    function NotifyOrderer(OrdererDUZ, ADFN, Instruments: string): string;
    function GetAdminID(aName: string; aDate: TDateTime): string;

    property Administrations: TObjectList read FAdministrations
      write FAdministrations;
    property ListOfIncompletes: TObjectList read FListOfIncompletes write FListOfIncompletes;
    property ProgressNoteStr: TStringList read FProgressNoteStr write FProgressNoteStr;
  end; // TAdministration_OPS


var
  Administration_OPS          : TAdministration_OPS;

implementation

uses
  Windows,
  Messages,
  uIA_Const,
  uMHA_Const,
  uIA_Patient,
  uIA_ThisUser,
  uIA_Question,
  uIA_Answer,
  uMHA_Answer_IO,
  uIA_Scale,
  uIA_Utils,
  uIA_Consult_IO,
  fIA_dialogs, Dialogs,
  fMHA_Main,
  uMHA_DM,
  uMHA_LocalFiles,
  uMHA_Report_OPS,
  uMHA_Report_IO,
  uMHA_Administration_IO;


constructor TAdministration_OPS.Create;
begin
  inherited Create;
  FAdministrations := TObjectList.Create;
  FListOfIncompletes := TObjectList.Create;
  FProgressNoteStr := TStringList.Create;
end;


destructor TAdministration_OPS.Destroy;
begin
//  if ListOfIncompletes <> nil then
  ListOfIncompletes.free;
//  if Administrations <> nil then
  Administrations.Free;
  FProgressNoteStr.Free;
  inherited Destroy;
end;

procedure TAdministration_OPS.CopyAdmin(aAdmin, bAdmin: TAdministration);
var
  i                           : integer;
  AAnswer                     : TAnswer;
begin
  bAdmin.ID := aAdmin.ID;
  bAdmin.InstrumentName := aAdmin.InstrumentName;
  bAdmin.AdministeredBy := aAdmin.AdministeredBy;
  bAdmin.OrdererDUZ := aAdmin.OrdererDUZ;
  bAdmin.RecordID := aAdmin.RecordID;
  bAdmin.PatientID := aAdmin.PatientID;
  bAdmin.Signed := aAdmin.Signed;
  bAdmin.DateSaved := aAdmin.DateSaved;
  bAdmin.DateGiven := aAdmin.DateGiven;
  bAdmin.IsComplete := aAdmin.IsComplete;
  bAdmin.NumberAnswered := aAdmin.NumberAnswered;
  bAdmin.DaysToRestart := aAdmin.DaysToRestart;
  bAdmin.IsRestartable := aAdmin.IsRestartable;
  bAdmin.LocationIEN := aAdmin.LocationIEN;
  bAdmin.ConsultIEN := aAdmin.ConsultIEN;
  bAdmin.IsaLegacy := aAdmin.IsaLegacy;
  bAdmin.GenerateProgNote := aAdmin.GenerateProgNote;
  bAdmin.ProgNoteTitle := aAdmin.ProgNoteTitle;
  bAdmin.ConsultTitle := aAdmin.ConsultTitle;

  bAdmin.ListOfAdminAnswers.Clear;
  for i := 0 to aAdmin.ListOfAdminAnswers.Count - 1 do
    begin
      AAnswer := TAnswer.Create;
      AAnswer.QuestionID := TAnswer(aAdmin.ListOfAdminAnswers[i]).QuestionID;
      AAnswer.AnswerValue := TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerValue;
      AAnswer.AnswerText := TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerText;
      AAnswer.Designator := TAnswer(aAdmin.ListOfAdminAnswers[i]).Designator;
      AAnswer.Designator := TAnswer(aAdmin.ListOfAdminAnswers[i]).Designator;
      AAnswer.AnswerLegacyValue := TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerLegacyValue;
      AAnswer.AnswerItemIndex := TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerItemIndex;
      AAnswer.AnswerChoiceID := TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerChoiceID;
      bAdmin.ListOfAdminAnswers.add(AAnswer);
    end;

end;


procedure TAdministration_OPS.ClearAdmin(aAdmin: TAdministration);
begin
  aAdmin.ID := NO_ID;
  aAdmin.RecordID := NO_ID;
  aAdmin.PatientID := Patient.DFN;
  aAdmin.Signed := False;
  aAdmin.DateSaved := Now;
  aAdmin.IsComplete := False;
  aAdmin.NumberAnswered := 0;
  aAdmin.DaysToRestart := 2;
  aAdmin.IsRestartable := False;
  aAdmin.ConsultIEN := '';
  aAdmin.IsaLegacy := false;
  aAdmin.GenerateProgNote := true;
  aAdmin.ProgNoteTitle := PROGRESS_NOTE_TITLE;
  aAdmin.ConsultTitle := CONSULT_NOTE_TITLE;
  aAdmin.ListOfAdminAnswers.Clear;

  // Do NOT clear FOrdererDUZ and
  // DateGiven, which are set in
  // InstrumentSelect and needs to stay constant
end;

procedure TAdministration_OPS.InitializeListOfAnswers(aInstrument: TInstrument;
  aAdmin: TAdministration);
var
  i                           : Integer;
  AAnswer                     : TAnswer;
begin
  aAdmin.ListOfAdminAnswers.Clear;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      AAnswer := TAnswer.Create;
      AAnswer.QuestionID := TQuestion(aInstrument.ListOfQuestions[i]).QuestionID;
      AAnswer.AnswerValue := NOT_ASKED;
      AAnswer.AnswerText := '';
      // CheckList is not considered multiple choice here
      AAnswer.IsMultiChoice := TQuestion(aInstrument.ListOfQuestions[i]).ResponseType = rtMultiChoice;
      AAnswer.Designator := TQuestion(aInstrument.ListOfQuestions[i]).Designator;
      AAnswer.AnswerLegacyValue := ' ';
      AAnswer.AnswerItemIndex := -1;
      AAnswer.AnswerChoiceID := '';
      aAdmin.ListOfAdminAnswers.Add(AAnswer);
    end; // for i
end;

procedure TAdministration_OPS.LoadAnswers(aInstrument: TInstrument;
  aAdmin: TAdministration);
begin

  InitializeListOfAnswers(aInstrument, aAdmin);

  if aAdmin.ID = NO_ID then // First administration of instrument
    Exit;

  // This is an incomplete administration; get previous answers
  if ReadAnswers(aAdmin, DataModule1.RPCBroker1) <> OK then // uMHA_Answers_IO
    ShowMessage('Unable to load answers.');

  if aAdmin.ListOfAdminAnswers.Count = 0 then
    begin
      MsgDlgFrm('There is a problem with the list of answers. This could be caused by ' +
        'a bad administration ID or errors with the API: ''YTQ ALL ANSWERS''. Recording ' +
        'answers will not be possible.', MHA_Caption, mtError, [mbOk], 0);
    end;
end;

function TAdministration_OPS.GetAdminID(aName: string; aDate: TDateTime): string;
var
  i                           : Integer;
begin
  result := '';

  if ListOfIncompletes.Count = 0 then
    Exit;

  for i := 0 to ListOfIncompletes.Count - 1 do
    begin
      if (TAdministration(ListOfIncompletes[i]).InstrumentName = aName) and
        (TAdministration(ListOfIncompletes[i]).DateGiven = aDate) then
        begin
          result := TAdministration(ListOfIncompletes[i]).ID;
          break;
        end;
    end;
end;

function TAdministration_OPS.DeleteAdmin(aAdminID: string): string;
begin
  Result := DeleteAdministration(aAdminID, DataModule1.RPCBroker1);
end;


function TAdministration_OPS.SaveAdmin(Admin: TAdministration): string;
begin
  Result := WriteAdministration(Admin, DataModule1.RPCBroker1);
end;

function TAdministration_OPS.SaveAnswers(Admin: TAdministration): Boolean;
begin
  Result := WriteAnswers(Admin, DataModule1.RPCBroker1); // uMHA_Answers_IO
end;

function TAdministration_OPS.NotifyOrderer(OrdererDUZ, ADFN, Instruments: string): string;
begin

  Result := OK;

  // Send notice only if the orderer is not the person logged on.
  // Notice is sent via Vista e-mail, so must be connected to Vista
  if (OrdererDUZ = ThisUser.DUZ) or
    (DatabaseMode <> dbVista) then
    Exit;

  Result := NotifyTestOrderer(OrdererDUZ, ADFN, Instruments, DataModule1.RPCBroker1);

end;


function TAdministration_OPS.SaveScore(aAdmin: TAdministration): string;
begin
  result := '';

  if DatabaseMode = dbOffLine then
    exit;

  if aAdmin.IsComplete then
    begin
      SaveThisScore(aAdmin.ID, DataModule1.RPCBroker1);

      (* HL7 was turned off Aug 2, 2011
      if aNationalTest then
        //Trigger sending an HL7 message
        if not SendHL7Message(aAdmin.ID, DataModule1.RPCBroker1) then
          Result := 'Unable to send HL7 message. ' + CRLF +
            'Data has been saved to Vista.';
       *)
    end;

end;

procedure TAdministration_OPS.SaveEditedConsultNote(aWriteFullText: Boolean;
  aAdmin: TAdministration);
var
  TempStr                     : string;
begin
  TempStr := WriteAdministrationConsultNote(
    aAdmin.ID,
    aAdmin.ConsultIEN,
    Piece(gvCoSignerIEN, U, 2), // Cosigner's IEN
    FProgressNoteStr,
    DataModule1.RPCBroker1);

  if TempStr = BAD_DATA then
    Exit;

  //"Refresh" CPRS data
  if gvCPRS_Instance_Handle <> 0 then
    PostMessage(gvCPRS_Instance_Handle, WM_COMMAND, 3, 0);
    //3= Refresh Patient info.
end;

procedure TAdministration_OPS.SaveConsultNote(
  aWriteFullText: Boolean;
  aAdmin: TAdministration);
var
  TempStr                     : string;
begin

  if aAdmin.IsaLegacy then
    //Read an existing Legacy Report
    FProgressNoteStr.Text := LegacyResultsReadTestReport2(Patient.DFN,
      aAdmin.ID, DataModule1.RPCBroker1)
  else
    BuildProgressNote(aAdmin, aWriteFullText);

  TempStr := WriteAdministrationConsultNote(
    aAdmin.ID,
    aAdmin.ConsultIEN,
    Piece(gvCoSignerIEN, U, 2), // Co signer's IEN
    FProgressNoteStr,
    DataModule1.RPCBroker1);

  if TempStr = BAD_DATA then
    Exit;

   //"Refresh" CPRS data
  if gvCPRS_Instance_Handle <> 0 then
    PostMessage(gvCPRS_Instance_Handle, WM_COMMAND, 3, 0);
    //3= Refresh Patient info.

end;


procedure TAdministration_OPS.SaveProgressNote(aWriteFullText: Boolean;
  aAdmin: TAdministration);
var
  TempStr                     : string;

begin
  if aAdmin.IsaLegacy then
    //Read an existing Legacy Report
    FProgressNoteStr.Text := LegacyResultsReadTestReport2(Patient.DFN,
      aAdmin.ID, DataModule1.RPCBroker1)
  else
    BuildProgressNote(aAdmin, aWriteFullText);

  TempStr := WriteAdministrationProgressNote(
    aAdmin.ID,
    gvProgressNoteIEN,
    Piece(gvCoSignerIEN, U, 2), // co-signer's IEN
    FProgressNoteStr,
    DataModule1.RPCBroker1);

  if TempStr = BAD_DATA then
    Exit;

  // Save only the first ProgressNoteIEN
  if gvProgressNoteIEN = '' then
    gvProgressNoteIEN := TempStr;

  //"Refresh" CPRS data
  if gvCPRS_Instance_Handle <> 0 then
    PostMessage(gvCPRS_Instance_Handle, WM_COMMAND, 3, 0);
    //3= Refresh Patient info.
end;

procedure TAdministration_OPS.SaveEditedProgressNote(aWriteFullText: Boolean;
  aAdmin: TAdministration);
var
  TempStr                     : string;
begin
  TempStr := WriteAdministrationProgressNote(aAdmin.ID,
    gvProgressNoteIEN,
    Piece(gvCoSignerIEN, U, 2),
    FProgressNoteStr,
    DataModule1.RPCBroker1);

  if TempStr = BAD_DATA then
    Exit;

  // Save only the first ProgressNoteIEN
  if gvProgressNoteIEN = '' then
    gvProgressNoteIEN := TempStr;

  //"Refresh" CPRS data
  if gvCPRS_Instance_Handle <> 0 then
    PostMessage(gvCPRS_Instance_Handle, WM_COMMAND, 3, 0);
    //3= Refresh Patient info.
end;


function TAdministration_OPS.GetItemIndex(aQuestionID, ChoiceID: string): string;
var
  j, i                        : Integer;
begin
  result := ChoiceID;

  if (ChoiceID = NOT_ASKED) or (ChoiceID = SKIPPED) or
    (ChoiceID = CHOICE_ID_OF_SKIPPED_QUESTION) or
    (ChoiceID = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION) or
    (ChoiceID = CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION) then
    Exit;

  for i := 0 to MHA_Report_OPS.ListOfQuestionsForReport.Count - 1 do
    if aQuestionID = TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).QuestionID then
      if TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).ResponseType = rtMultiChoice then
        for j := 0 to TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).MultiChoice.choices.Count - 1 do
          if Piece(TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).MultiChoice.choices[j], U, 2) =
            ChoiceID then
            result := IntToStr(j);
end;



function TAdministration_OPS.BuildProgressNote(aAdmin: TAdministration;
  aWriteFullText: Boolean): string;
var
  i                           : Integer;
  ReportID,
    AnswerString,
    ResultString              : string;
  dllHandle                   : THandle;


  procedure HandleGroomedAnswers;
  var
    ii                        : Integer;
    TempStr                   : string;
  begin
    // AnswerString format:
    // QuestionID=ItemIndex^AnswerText^AnswerID

{
   Patch 105 llh - way it was... we need to be looking at MHA_Report_OPS
                   answers have been filed in VistA, need to run as if a report
                   not an administration.

    for ii := 0 to aAdmin.ListOfAdminAnswers.Count - 1 do
      AnswerString := AnswerString +
        TAnswer(aAdmin.ListOfAdminAnswers[ii]).QuestionID + EQ +
        GetItemIndex(TAnswer(aAdmin.ListOfAdminAnswers[ii]).QuestionID,
        TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerValue) + U +
        TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerValue + U +
        TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerID +
        RECORD_END;
}

    for ii := 0 to MHA_Report_OPS.ListOfAnswersForReport.Count - 1 do
      AnswerString := AnswerString +
        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[ii]).QuestionID + EQ +
        GetItemIndex(TAnswer(MHA_Report_OPS.ListOfAnswersForReport[ii]).QuestionID,
        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[ii]).AnswerValue) + U +
        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[ii]).AnswerValue + U +
        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[ii]).AnswerID +
        RECORD_END;

    if Assigned(GroomAnswersDLL) then
      GroomAnswersDLL(
        aAdmin.InstrumentName,
        Patient.Name,
        Patient.DOB,
        Patient.Sex,
        AnswerString,
        ResultString);

    if (ResultString = '') or
      (ResultString = AnswerString) then
      begin
        ResultString := '';
        Exit;
      end
    else
      begin
          // Place new answer values in current list of answer values.
        AnswerString := '';
        for ii := 0 to aAdmin.ListOfAdminAnswers.Count - 1 do
          begin
            TempStr := Piece(ResultString, RECORD_END, i);
            TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerValue :=
              Piece(Piece(TempStr, EQ, 2), U, 1);
            TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerText :=
              Piece(Piece(TempStr, EQ, 2), U, 2);

            AnswerString := AnswerString +
              TAnswer(aAdmin.ListOfAdminAnswers[ii]).QuestionID + EQ +
              GetItemIndex(TAnswer(aAdmin.ListOfAdminAnswers[ii]).QuestionID,
              TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerValue) + U +
              TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerValue + U +
              TAnswer(aAdmin.ListOfAdminAnswers[ii]).AnswerID + RECORD_END;
          end;
      end; // else
  end;

begin
  FProgressNoteStr.Clear;

  if not MHA_Report_OPS.BuildAllReportLists(aAdmin.InstrumentName,
    aAdmin.ID) then
    Exit;

  AnswerString := '';
  ResultString := '';
  ReportID := '0';
  result := '';

  // patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
    // patch 105 added gvAuxFile, removed C:\Program Files llh
    MsgDlgFrm('Could not find DLL used for scoring instruments.' +
      'The file YS_MHA_AUX.DLL is usually located in ' + gvAuxFile, MHA_CAPTION, mtWarning, [mbOk], 0)
  else
    begin
      try
        @GroomAnswersDLL := GetProcAddress(dllHandle, 'GroomAnswers');

        if @GroomAnswersDLL = nil then
             // function not found, continue below
        else
          HandleGroomedAnswers;
      finally
        FreeLibrary(dllHandle);
      end; {try..finally}
    end; // if dllHandle

  // patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  // Check for hard coded scoring to add to report
  if dllHandle = 0 then // do nothing
  else
    begin
      try
        @ProgressNoteDLL := GetProcAddress(dllHandle, 'ProgressNote');

 //           for i := 0 to MHA_Report_OPS.ListOfAnswersForReport.Count - 1 do
//                      AnswerString := AnswerString +
//                        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[i]).QuestionID + EQ +
//                        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[i]).AnswerValue + U +
//                        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[i]).AnswerText + U +
//                        TAnswer(MHA_Report_OPS.ListOfAnswersForReport[i]).AnswerID + RECORD_END;
//            ShowMessage(answerstring);

        if @ProgressNoteDLL = nil then //do nothing
        else
          if Assigned(ProgressNoteDLL) then
            ProgressNoteDLL(aAdmin.InstrumentName,
              Patient.Name,
              Patient.DOB,
              Patient.Sex,
              AnswerString,
              ResultString);
      finally
        FreeLibrary(dllHandle);
      end; {try..finally}

    end; // if dllHandle

  if MHA_Report_OPS.LoadReport(
    aAdmin.InstrumentName,
    aAdmin.ID,
    aAdmin.AdministeredBy,
    aAdmin.OrdererDUZ,
    aAdmin.LocationIEN,
    DateToStr(aAdmin.DateGiven),
    DateToStr(aAdmin.DateSaved),
    0, // TestIENS, (important for legacy test, not used here)
    True, // ForceWrap; Used for progress notes
    aAdmin.IsaLegacy, // non legacy, handled already
    aWriteFullText,
    ResultString, // from YS_MHA_AUX.dll if used
    FProgressNoteStr) = 'ERROR' then
    begin
 //     MsgDlgFrm('An error has occured while generating this report.',
 //       MHA_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;

end;

  (*
procedure TAdministration_OPS.LoadNewOrderer(ADUZ: string);
var
  tmpDUZ                      : string;
begin
  tmpDUZ := ADUZ;
  Orderer.Load(tmpDUZ);
end; *)

(*
procedure TAdministration_OPS.LoadIncompleteInstance(ARecord: string;
  AIsRestartable: BOOLEAN);
//   1         2                  3           4             5    6   7 8 9 10 11   12 1314
//100764^A Report-Script Test^3050317.13525^3050317.155349^6627^6627^N^11^^N^100065^^N^2~
//100750^A Pretty Good Test^3050314.063229^3050314.083402^6627^6627^N^11^^N^100038^^N^12~

//1. Administration ID
//2. Instrument Name
//3. Date Given
//4. Date Saved
//5. Orderer
//6. Admin.By
//7. Signed
//8. # Answers
//9. Exempt/ R_Privl
//10. Is Legacy
//11. INSTRUMENT id 601.71
//12. Test IENS 601
//13. Is Copyrighted
//14. Location IEN

begin

  if (ARecord = '') then
    Exit;

  FPatientID := Patient.DFN;
  FIsComplete := False;

  FRecordID := Piece(ARecord, EQ, 1);
  ARecord := Piece(ARecord, EQ, 2);
  FID := Piece(ARecord, U, 1);
  FInstrumentName := Piece(ARecord, U, 2);

  FDateGiven := StrToDateTime(FMDateTimeStrToDateTimeStr(Piece(ARecord, U, 3)));
  FDateSaved := StrToDateTime(FMDateTimeStrToDateTimeStr(Piece(ARecord, U, 4)));

  Orderer.DUZ := Piece(ARecord, U, 5);
  FAdministeredBy := Piece(ARecord, U, 6);

  //A restarted instrument will never be signed, and FSigned is set to false
  FSigned := false;

  FNumberAnswered := StrToIntDef(Piece(ARecord, U, 8), 0);
  FIsRestartable := AIsRestartable;
  FLocationIEN := Piece(ARecord, U, 14);

  //Exempt, IsLegacy and IENS are properties of TInstrument, not TAdministration,
  //and so pieces 10-12 of ARecord are ignored.
  //Those pieces are used by InstrumentResults
end;  *)


procedure TAdministration_OPS.ReadIncompleteResults(aDFN, OrdererDUZ: string);
begin
  case DataBaseMode of
    dbVista:
      begin
        if ReadAdministrations(aDFN, OrdererDUZ, DataModule1.RPCBroker1) <> OK then // see uMHA_Administration_IO
          ShowMessage('Error reading previous administrations.');
      end; //dbVista

    dbOffLine:
      ReadOffLineAdministrations(aDFN); // see uMHA_Administration_IO

  end; //case DataBaseMode
end;

function TAdministration_OPS.ReadCrashFile(aDFN, aDUZ, aInstrName: string): boolean;
var
  i, NumAnswered              : Integer;
  TempStr                     : string;
  RecoveryStrList             : TStringList;
  aAdmin                      : TAdministration;
  aAnswer                     : TAnswer;
begin
  Result := True;
  RecoveryStrList := TStringList.Create;

  // The file has already been verified as existing
  RecoveryStrList.LoadFromFile(LocalFiles.AnswerFilesPath + '\' +
    aDFN + FILE_NAME_DELIMITER + aDUZ + FILE_NAME_DELIMITER + aInstrName + '.' +
    CRASH_FILE_EXT);

  if RecoveryStrList.Count = 0 then
    Exit;

  NumAnswered := 0;
  TempStr := RecoveryStrList[0];

  if (TempStr = '') or (NumberOfPieces(TempStr, U) <> 7) then
    Exit;

  aAdmin := TAdministration.Create;

  with aAdmin do
    begin
      ID := NO_ID;
      InstrumentName := Piece(TempStr, U, 1);
      AdministeredBy := Piece(TempStr, U, 2);
      PatientID := Piece(TempStr, U, 3);
      OrdererDUZ := Piece(TempStr, U, 4);
      LocationIEN := Piece(TempStr, U, 5);
      LocationStr := Piece(TempStr, U, 6);
      DateGiven := StrToDateTime(Piece(TempStr, U, 7));
      Signed := False;
      IsComplete := False;

      for i := 1 to RecoveryStrList.Count - 1 do
        begin
          aAnswer := TAnswer.Create;
          TempStr := RecoveryStrList[i];
          with aAnswer do
            begin
              AnswerID := '';
              AnswerVUID := '';
              QuestionID := Piece(TempStr, U, 1);
              Designator := Piece(TempStr, U, 2);
              AnswerValue := Piece(TempStr, U, 3);
              AnswerText := Piece(TempStr, U, 4);
              AnswerLegacyValue := Piece(TempStr, U, 5);
              IsMultiChoice := StrToBool(Piece(TempStr, U, 6));
              AnswerItemIndex := StrToIntDef(Piece(TempStr, U, 7), -1);
              AnswerChoiceID := Piece(TempStr, U, 8);
              if AnswerValue <> NOT_ASKED then
                Inc(NumAnswered);
            end; // with aAnswer
          aAdmin.NumberAnswered := NumAnswered;
          aAdmin.ListOfAdminAnswers.Add(aAnswer);
        end; // for i

      TempStr := SaveAdmin(aAdmin);
      aAdmin.ID := TempStr;
      Result := WriteAnswers(aAdmin, DataModule1.RPCBroker1);
      Administration_OPS.ListOfIncompletes.Add(aAdmin);
    end; // with aAdmin

  RecoveryStrList.Free;
end;

initialization
  Administration_OPS := TAdministration_OPS.Create;

finalization
  Administration_OPS.Free;

end.

