unit uMHA_Registry;

interface

uses
  Registry,
  SysUtils,
  System.UITypes,
  Forms,
  fIA_dialogs, Dialogs,
  Windows,
  WinProcs,
  WinTypes,
  graphics;


type

  TMHA_Registry = class(TObject)
  private
    function CheckForFilesAndValues: Boolean;
    procedure SetFont(Value: TFont);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure SaveRegistry;
    function ReadRegistry: boolean;
    procedure UseDefaultRegistry;
  //published
  end;

var
  MHA_Registry                : TMHA_Registry;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils;

const
  WindowsRegistry             : string = 'SOFTWARE\VISTA\YS\MHA3\';

constructor TMHA_Registry.Create;
begin
  inherited Create;
  gvShowOptionTabs := '012'; // show all tabs (default)
end;

destructor TMHA_Registry.Destroy;
begin
  inherited Destroy;
end;

function SetFontToString(FontIn: TFont): string;
begin
  Result := '';
  with FontIn do
    begin
      Result := Name + U +
        IntToStr(Color) + U +
        IntToStr(PixelsPerInch) + U +
        IntToStr(Size) + U;
      if Style = [fsBold] then
        Result := Result + 'B' + U
      else
        Result := Result + U;
      if Style = [fsItalic] then
        Result := Result + 'I' + U
      else
        Result := Result + U;
      if Style = [fsUnderline] then
        Result := Result + 'U' + U
      else
        Result := Result + U;
      if Style = [fsStrikeout] then
        Result := Result + 'S' + U
      else
        Result := Result + U;
      if Pitch = fpDefault then
        Result := Result + 'D' + U;
      if Pitch = fpFixed then
        Result := Result + 'F' + U;
      if Pitch = fpVariable then
        Result := Result + 'V' + U;
    end; // with
end;

procedure SetStringToFont(StringIn: string);
var
  TempFont                    : TFont;
begin
  TempFont := TFont.Create;
  with TempFont do
    begin
      if (Piece(StringIn, U, 1) = '') or
        (NumberOfPieces(StringIn, U) <> 10) then
        begin
          Name := 'MS Sans Serif';
          Color := clBlack;
          PixelsPerInch := 96;
          Size := 10;
          Style := [];
          Pitch := fpDefault;
        end
      else
        begin
          Name := Piece(StringIn, U, 1);
          Color := StrToInt(Piece(StringIn, U, 2));
          PixelsPerInch := StrToInt(Piece(StringIn, U, 3));
          Size := StrToInt(Piece(StringIn, U, 4));
          //llh patch 129 -- ensure font size is readable
          if Size = 0 then
            Size := 8;
          Style := [];
          if Piece(StringIn, U, 5) = 'B' then
            Style := Style + [fsBold];
          if Piece(StringIn, U, 6) = 'I' then
            Style := Style + [fsItalic];
          if Piece(StringIn, U, 7) = 'U' then
            Style := Style + [fsUnderline];
          if Piece(StringIn, U, 8) = 'S' then
            Style := Style + [fsStrikeout];
          Pitch := fpDefault;
          if Piece(StringIn, U, 9) = 'V' then
            Pitch := fpVariable;
          if Piece(StringIn, U, 9) = 'F' then
            Pitch := fpFixed;
        end; // with

      gvFont.Assign(TempFont);
      TempFont.Free;
    end;
end;

function TMHA_Registry.ReadRegistry: boolean;
var
  ReadReg                     : TRegistry;
begin
  Result := False;
  ReadReg := TRegistry.Create;

  try
    with ReadReg do
      begin
        RootKey := HKEY_CURRENT_USER; // better for Thin client applications
        if OpenKey(WindowsRegistry, true) then
          begin
            if not ValueExists('DLL_Version') then
              begin
                CloseKey;
                Free;
                UseDefaultRegistry;
                SaveRegistry;
                Result := True;
                exit;
              end;

            // location of Main form on user's screen
            gvLeft := ReadInteger('UserLeft');
            gvTop := ReadInteger('UserTop');
            gvHeight := ReadInteger('UserHeight');
            gvWidth := ReadInteger('UserWidth');
            gvLeftMargin := ReadInteger('LeftMargin');
            gvRightMargin := ReadInteger('RightMargin');
            gvTopMargin := ReadInteger('TopMargin');
            gvBottomMargin := ReadInteger('BottomMargin');
            gvASIWidth := ReadInteger('ASIWidth');
            gvSelectWidth := ReadInteger('SelectWidth');
            gvResultWidth := ReadInteger('ResultWidth');
            gvGAFWidth := ReadInteger('GAFWidth');
            gvSingleWidth := ReadInteger('SingleWidth');
            gvASIHeight := ReadInteger('ASIHeight');
            gvSelectHeight := ReadInteger('SelectHeight');
            gvResultHeight := ReadInteger('ResultHeight');
            gvGAFHeight := ReadInteger('GAFHeight');
            gvSingleHeight := ReadInteger('SingleHeight');

            gvSpeedTab := ReadBool('SpeedTab');
            gvVisualBeeps := ReadBool('VisualBeeps');
            gvDefaultSize := ReadBool('DefaultSize');
            gvShowHints := ReadBool('ShowHints');
            gvShowImage := ReadBool('ShowImage');
            gvUseDesignFont := ReadBool('UseDesignFont');
            gvShowAllOrOneItem := ReadBool('ShowAllOrOneItem');
            gvUseScreenReader := ReadBool('UseScreenReader');
            gvShowScreenReaderMsg := ReadBool('ShowScreenReaderMsg');

            gvHighlightForeground := ReadInteger('UserColorForeground');
            gvHighlightBackground := ReadInteger('UserColorBackground');
            // Location of splitter, user defined
            gvASIGeneral := ReadInteger('ASIGeneral');
            gvASIMedical := ReadInteger('ASIMedical');
            gvASIEmploy := ReadInteger('ASIEmploy');
            gvASIDrug := ReadInteger('ASIDrug');
            gvASILegal := ReadInteger('ASILegal');
            gvASIFamily := ReadInteger('ASIFamily');
            gvASISocial := ReadInteger('ASISocial');
            gvASIPsych := ReadInteger('ASIPsych');
            gvStart := ReadInteger('StartMenu');
            SetStringToFont(ReadString('DefaultFont'));
            gvLastASIProgram := ReadInteger('LastASIProgram');
            gvLastLocation := ReadString('LastLocation');
            gvUpdate := StrToDate(ReadString('Update'));
            //gvJAWS_Dir := ReadString('JAWS_Directory');
            gvSpecialResultsList := ReadString('HealthSumList');
            gvCoSignerIEN := ReadString('Cosigner');
            gvDLL_Version := ReadString('DLL_Version');
            // patch 105 add getting gvEnvVar to be used in determining location of YS_MHA_AUX.dll llh
            gvEnvVar := GetEnvironmentVariable('PROGRAMFILES') ;
          end;

        CloseKey;
        Free;
      end; {- with ReadReg -}
  except
    on E: exception do
      begin
        // Load default values so program will load
        UseDefaultRegistry;
      end;
  end; {- try -}

  if not CheckForFilesAndValues then
    exit;
  result := True;
end;

procedure TMHA_Registry.SaveRegistry;
var
  SaveReg                     : TRegistry;

begin
  SaveReg := TRegistry.Create;
  try
    with SaveReg do
      begin
        RootKey := HKEY_CURRENT_USER; // better for Thin client applications
        if OpenKey(WindowsRegistry, true) then
          begin
            WriteInteger('UserLeft', gvLeft);
            WriteInteger('UserTop', gvTop);
            WriteInteger('UserHeight', gvHeight);
            WriteInteger('UserWidth', gvWidth);
            WriteInteger('LeftMargin', gvLeftMargin);
            WriteInteger('RightMargin', gvRightMargin);
            WriteInteger('TopMargin', gvTopMargin);
            WriteInteger('BottomMargin', gvBottomMargin);
            WriteBool('SpeedTab', gvSpeedTab);
            WriteBool('VisualBeeps', gvVisualBeeps);
            WriteBool('ShowHints', gvShowHints);
            WriteBool('ShowImage', gvShowImage);
            WriteBool('DefaultSize', gvDefaultSize);
            WriteBool('UseDesignFont', gvUseDesignFont);
            WriteBool('ShowAllOrOneItem', gvShowAllOrOneItem);
            WriteBool('UseScreenReader', gvUseScreenReader);
            WriteBool('ShowScreenReaderMsg', gvShowScreenReaderMsg);

            WriteInteger('UserColorForeground', gvHighlightForeground);
            WriteInteger('UserColorBackground', gvHighlightBackground);
            WriteInteger('ASIGeneral', gvASIGeneral);
            WriteInteger('ASIMedical', gvASIMedical);
            WriteInteger('ASIEmploy', gvASIEmploy);
            WriteInteger('ASIDrug', gvASIDrug);
            WriteInteger('ASILegal', gvASILegal);
            WriteInteger('ASIFamily', gvASIFamily);
            WriteInteger('ASISocial', gvASISocial);
            WriteInteger('ASIPsych', gvASIPsych);

            WriteInteger('ASIWidth', gvASIWidth);
            WriteInteger('SelectWidth', gvSelectWidth);
            WriteInteger('ResultWidth', gvResultWidth);
            WriteInteger('GAFWidth', gvGAFWidth);
            WriteInteger('SingleWidth', gvSingleWidth);
            WriteInteger('ASIHeight', gvASIHeight);
            WriteInteger('SelectHeight', gvSelectHeight);
            WriteInteger('ResultHeight', gvResultHeight);
            WriteInteger('GAFHeight', gvGAFHeight);
            WriteInteger('SingleHeight', gvSingleHeight);

            WriteInteger('UserFontColor', gvFont.Color);
            WriteInteger('StartMenu', gvStart);
            WriteInteger('LastASIProgram', gvLastASIProgram);

            WriteString('LastLocation', gvLastLocation);
            WriteString('Update', DateToStr(gvUpdate));
            //WriteString('JAWS_Directory', gvJAWS_Dir);
            WriteString('HealthSumList', gvSpecialResultsList);
            WriteString('Cosigner', gvCoSignerIEN);
            WriteString('DefaultFont', SetFontToString(gvFont));
            WriteString('DLL_Version', gvDLL_Version);

          end;
        CloseKey;

        Free;
      end; {- with SaveReg -}
  except
    on E: Exception do
      begin
        exit;
      end;
  end; {- try-}
end;

procedure TMHA_Registry.UseDefaultRegistry;
var
  NonClientMetrics            : TNonClientMetrics;
begin

  if (Screen.Width = 640) then
    begin
      gvTop := 3;
      gvLeft := 3;
      gvWidth := 630;
      gvHeight := 470;
    end
  else
    begin
      gvTop := 5;
      gvLeft := 5;
      gvWidth := 790;
      gvHeight := 590;
    end;

  gvASIGeneral := 80;
  gvASIMedical := 80;
  gvASIEmploy := 80;
  gvASIDrug := 80;
  gvASILegal := 80;
  gvASIFamily := 80;
  gvASISocial := 80;
  gvASIPsych := 80;

  gvASIWidth := gvWidth;
  gvSelectWidth := gvWidth;
  gvResultWidth := gvWidth;
  gvGAFWidth := gvWidth;
  gvSingleWidth := gvWidth;
  gvASIHeight := gvHeight;
  gvSelectHeight := gvHeight;
  gvResultHeight := gvHeight;
  gvGAFHeight := gvHeight;
  gvSingleHeight := gvHeight;

  gvLeftMargin := 96; // Pixels: One inch is 96 pixels for DPI 96.
  gvRightMargin := 96;
  gvTopMargin := 96;
  gvBottomMargin := 96;

  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    gvFont.Handle := CreateFontIndirect(NonClientMetrics.lfMessageFont);
  SetFont(gvFont);

  gvSpeedTab := True;
  gvVisualBeeps := False;
  gvDefaultSize := True;
  gvShowHints := True;
  gvShowImage := True;
  gvUseScreenReader := False;
  gvShowScreenReaderMsg := True;
  gvHighlightForeground := clBlack;
  gvHighlightBackground := clYellow;
  gvUseDesignFont := False;
  gvShowAllOrOneItem := True;
  gvLastASIProgram := 0;
  gvLastLocation := '-1^';
  gvUpdate := Date;
  //gvJAWS_Dir := 'c:\documents and settings\all users\application data\freedom scientific\jaws\10.0\settings\enu\YS_MHA.JCF';
  // Do not over write user's list of Special Results
  //gvSpecialResultsList := 'AUDC^G~PHQ9^G~';
  gvCoSignerIEN := '';
  gvStart := 0;
  gvDLL_Version := '1.0.3.2'; // do not use blank here
end;

function TMHA_Registry.CheckForFilesAndValues: boolean;
// This will ensure that all the proper files are present.

  function CheckColor(ColorIn, DefaultColor: TColor): TColor;
  begin
    Result := DefaultColor;
    if ((ColorIn = clYellow) or (ColorIn = clBlue) or (ColorIn = clBlack) or
      (ColorIn = clGray) or (ColorIn = clDkGray) or (ColorIn = clGreen) or
      (ColorIn = clLime) or (ColorIn = clMaroon) or (ColorIn = clNavy) or
      (ColorIn = clOlive) or (ColorIn = clPurple) or (ColorIn = clRed) or
      (ColorIn = clSilver) or (ColorIn = clTeal) or (ColorIn = clAqua) or
      (ColorIn = clFuchsia) or (ColorIn = clWhite)) then
      Result := ColorIn;
  end;

begin

  gvHighlightForeground := CheckColor(gvHighlightForeground, clBlack);
  gvHighlightBackground := CheckColor(gvHighlightBackground, clYellow);
  if gvHighlightForeground = gvHighlightBackground then
    begin
      gvHighlightForeground := clBlack;
      gvHighlightBackground := clYellow;
    end;

  SetFont(gvFont);

  // Some fonts are too big for 640x480 & 800x600
  if ((Screen.Width = 640) and (gvFont.Size >= 14)) then
    begin
      MsgDlgFrm('The font you have selected is too large for your ' +
        '640x480 screen area. Either enlarge the screen area in ' +
        'Start|Control Panel|Display|Settings|Screen Area, or use a smaller font.',
        MHA_Caption, mtInformation, [mbOK], 0);
      gvFont.Size := 12;
    end;
  if ((Screen.Width = 800) and (gvFont.Size >= 18)) then
    begin
      MsgDlgFrm('The font you have selected is too large for your ' +
        '800x600 screen area. Either enlarge the screen area in ' +
        'Start|Control Panel|Display|Settings|Screen Area, or use a smaller font.',
        MHA_Caption, mtInformation, [mbOK], 0);
      gvFont.Size := 14;
    end;
  Result := True;
end;


procedure TMHA_Registry.SetFont(Value: TFont);
begin
  gvFont.Assign(Value);
end;

initialization
  MHA_Registry := TMHA_Registry.Create;


finalization
  MHA_Registry.Free;

end.

