unit uMHA_Report_IO;

interface

uses
  SysUtils,
  System.Contnrs,
  fIA_dialogs, Dialogs,
  Classes,
  Trpcb;

function ReadReportTemplate(aInstrument: string;
  var ReportIEN: string; Broker: TRPCBroker): string;
function LoadAnswersForReport(AdministrationID, ADFN: string; Broker: TRPCBroker): string;
function LoadQuestionsForReport(aInstrument: string; Broker: TRPCBroker): string;
function LoadQuestionChoicesForReport(aInstrument: string; Broker: TRPCBroker): string;
function LoadScalesForReport(aInstrument: string; Broker: TRPCBroker): string;
function ReadScoringKeys(aInstrument: string; Broker: TRPCBroker): Boolean;

//patch 123 load Instrument Results for a specific administration
function LoadAdminResults(AdministrationID: string; Broker: TRPCBroker) : string;

//function LoadAnswersForSummary(AdministrationID: string; Broker: TRPCBroker): string;
//function GetNotesList(DFN: string; Broker: TRPCBroker): TStrings;
function GetPatientInfo(DFN: string; Broker: TRPCBroker): TStrings;

//Legacy reports
function LegacyResultsReadTestReport1(ADFN, YSText: string; Broker: TRPCBroker): string;
function LegacyResultsReadTestReport2(ADFN, aAdminID: string; Broker: TRPCBroker):
  string;

//Comments
function ReadComments(aAdminID: string; Broker: TRPCBroker): string;
function WriteComments(aAdminID, AComment: string; Broker: TRPCBroker): string;

//function GetLegacyScores(Broker: TRPCBroker;
//  aInstrument,
//  aMDate,
//  aPatientDFN,
//  aOrderedByDUZ,
//  aLegacyAnswerString: string): string;

//function LocationIndexToName(IndexIn: string): string;
//function GetLegacyReport(Broker: TRPCBroker;
//  aInstrument,
//  aMDate,
//  aPatientDFN,
//  aOrderedByDUZ,
//  aLegacyAnswerString: string): string;

implementation

uses
  uIA_Const,
  uIA_Answer,
  uIA_Question,
  uIA_Question_OPS,
  uIA_Scale,
  uIA_Key,
  uIA_MultiChoice_IO,
  uMHA_Report_OPS,
  uMHA_LocalFiles,
  uIA_Utils;


function ReadReportTemplate(aInstrument: string;
  var ReportIEN: string; Broker: TRPCBroker): string;
var
  i,
    ItemCount                 : Integer;
begin
  Result := '';
  ReportIEN := '0';

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ GET REPORT: ' +
      //*   ' FILEN=601.93' +
      //*   ' CODE=' + aInstrument);
      RemoteProcedure := 'YTQ GET REPORT';
      Param[0].Mult['"FILEN"'] := '601.93';
      Param[0].Mult['"CODE"'] := aInstrument;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GET REPORT: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GET REPORT: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      ItemCount := Results.count;

      if POS('[DATA]', Results[0]) > 0 then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ GET REPORT: Successful');
          ReportIEN := Piece(Results[0], U, 2);

          for i := 1 to ItemCount - 1 do
            Result := Result + Results[i];
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ GET REPORT: Error ');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR';
        end;

         // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end;
end;

(*
function GetLegacyScores(Broker: TRPCBroker;
  aInstrument,
  aMDate,
  aPatientDFN,
  aOrderedByDUZ,
  aLegacyAnswerString: string): string;
var
  i                           : Integer;
  Scores                      : Boolean;
begin
  result := '';
  scores := True;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ LEGCR';

      Param[0].Mult['"CODE"'] := aInstrument;
      Param[0].Mult['"ADATE"'] := aMDate;
      Param[0].Mult['"DFN"'] := aPatientDFN;
      Param[0].Mult['"STAFF"'] := aOrderedByDUZ;
      Param[0].Mult['"R1"'] := Copy(aLegacyAnswerString, 1, 200);
      Param[0].Mult['"R2"'] := Copy(aLegacyAnswerString, 201, 400);
      Param[0].Mult['"R3"'] := Copy(aLegacyAnswerString, 401, 600);
      Param[0].PType := List;

      Call;

      if Results.Count < 10 then
        begin
          Result := 'ERROR in API call: YTQ LEGCR';
          Exit;
        end;

      // Each scale and its score has a line in results starting at 5
      // 1^Scale1 name^ Scale1 Score
      // 2^Scale2 name^ Scale2 score ...

      for i := 5 to Results.Count - 1 do
        begin
          if Results[i] = '^^PROGRESS NOTE^^' then
            begin
              results[i] := '^';
              scores := False;
            end;
          if Scores then
            result := result + '*' + piece(results[i], U, 3) + FILE_NAME_DELIMITER +
              piece(results[i], U, 4);
          //else
            //result := result + results[i] + FILE_NAME_DELIMITER {~};
        end;
    end; // Broker
end;   *)


function LegacyResultsReadTestReport1(ADFN, YSText: string; Broker: TRPCBroker):
  string;
const
  ERRORMSG                    = 'Unable to retrieve test report.' + CRLF
    + 'Error in LegacyResultsReadTestReport1' + CRLF + CRLF;
var
  i                           : integer;
begin

  Result := '';

  with Broker do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTRP INSTRUMENT REPORT: AD=' + YSText);
      RemoteProcedure := 'YTRP INSTRUMENT REPORT';
      Param[0].Mult['"AD"'] := YSText;
      Param[0].PType := list;
      Param[0].Value := ADFN;
      Param[0].PType := literal;
      Param[1].Value := YSText;
      Param[1].PType := literal;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            Result := ERRORMSG + 'Broker Error: ' + E.Message;
            //* LocalFiles.UpdateAPILog('Call to YTRP INSTRUMENT REPORT: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + result);
            Exit;
          end;

        on E: EAccessViolation do
          begin
            Result := ERRORMSG + 'Access Violation: ' + E.Message;
            //* LocalFiles.UpdateAPILog('Call to YTRP INSTRUMENT REPORT: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + result);
            Exit;
          end;

      end; //try
      //* LocalFiles.UpdateAPILog('Call to YTRP INSTRUMENT REPORT: Successful');
      for i := 1 to Results.Count - 1 do
        Result := Result + Results[i] + CRLF;
        
     // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; //with DataModule1...

  Result := DropTrailer(Result);

end;

function LegacyResultsReadTestReport2(ADFN, aAdminID: string; Broker: TRPCBroker):
  string;
var
  i                           : integer;
const
  ERRORMSG                    = 'Unable to retrieve test report.' + CRLF
    + 'Error in LegacyResultsReadTestReport2' + CRLF + CRLF;
begin

  Result := '';

  with Broker do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ LEGACY REPORT: AD=' + aAdminID);
      RemoteProcedure := 'YTQ LEGACY REPORT';
      Param[0].Mult['"AD"'] := aAdminID;
      Param[0].PType := list;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            Result := ERRORMSG + 'Broker Error: ' + E.Message;
            //* LocalFiles.UpdateAPILog('Call to YTQ LEGACY REPORT: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + result);
            Exit;
          end;

        on E: EAccessViolation do
          begin
            Result := ERRORMSG + 'Access Violation: ' + E.Message;
            //* LocalFiles.UpdateAPILog('Call to YTQ LEGACY REPORT: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + result);
            Exit;
          end;

      end; //try

     for i := 1 to Results.Count - 1 do
        Result := Result + Results[i] + CRLF;
      //* LocalFiles.UpdateAPILog('Call to YTQ LEGACY REPORT: Successful');

       // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end;

  Result := DropTrailer(Result);

end;


function ReadComments(aAdminID: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  Result := '';

  with Broker do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE:' +
      //*   ' FILEN=601.84' +
      //*   ' FIELDS=10' +
      //*   ' IENS=' + aAdminID);
      RemoteProcedure := 'YTQ CAPIE';
      //Param[0].Mult['"FILEN"'] := FILE_ADMINISTRATIONS;
      Param[0].Mult['"FILEN"'] := '601.84';
      Param[0].Mult['"FIELDS"'] := '10';
      Param[0].Mult['"IENS"'] := aAdminID;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if not (Results[0] = DATA) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Error ' + Results[0]);
          Result := BAD_DATA;
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Successful');
          for i := 1 to Results.Count - 1 do
            Result := Result + Piece(Results[i], U, 3);
        end;
    end;
end;

function WriteComments(aAdminID, AComment: string; Broker: TRPCBroker): string;
var
  i                           : integer;
  tmpComment, YS              : string;
begin

  tmpComment := AComment;
  tmpComment := StringToPieces(tmpComment, U, 200);

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: ' +
      //*   'FILEN=601.84' +
      //*   'IEN=' + aAdminID +
      //*   'FIELD=10');
      RemoteProcedure := 'YTQ WP FILER';
      //Param[0].Mult['"FILEN"'] := FILE_ADMINISTRATIONS;
      Param[0].Mult['"FILEN"'] := '601.84';
      Param[0].Mult['"IEN"'] := aAdminID;
      Param[0].Mult['"FIELD"'] := '10';

      for i := 1 to NumberOfPieces(tmpComment, U) do
        begin
          YS := '"' + IntToStr(i) + '"';
          Param[0].Mult[YS] := Piece(tmpComment, U, i);
        end;

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ' + Results[1]);
          Result := 'ERROR: YTQ WP FILER: 601.84 FAILED '
            + U + Results[1];
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Successful');
          Result := OK;
        end;
    end;

end;

(*
function GetLegacyReport(Broker: TRPCBroker;
  aInstrument,
  aMDate,
  aPatientDFN,
  aOrderedByDUZ,
  aLegacyAnswerString: string): string;
var
  i                           : Integer;
  Scores                      : Boolean;
begin
  result := '';
  scores := True;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      RemoteProcedure := 'YTQ LEGCR';

      Param[0].Mult['"CODE"'] := aInstrument;
      Param[0].Mult['"ADATE"'] := aMDate;
      Param[0].Mult['"DFN"'] := aPatientDFN;
      Param[0].Mult['"STAFF"'] := aOrderedByDUZ;
      Param[0].Mult['"R1"'] := Copy(aLegacyAnswerString, 1, 200);
      Param[0].Mult['"R2"'] := Copy(aLegacyAnswerString, 201, 400);
      Param[0].Mult['"R3"'] := Copy(aLegacyAnswerString, 401, 600);
      Param[0].PType := List;

      Call;

      if Results.Count < 10 then
        begin
          Result := 'ERROR in API call: YTQ LEGCR';
          Exit;
        end;

      // Each scale and its score has a line in results starting at 5
      // 1^Scale1 name^ Scale1 Score
      // 2^Scale2 name^ Scale2 score ...

      result := ' SCORE(S) = ';

      for i := 5 to Results.Count - 1 do
        begin
          if Results[i] = '^^PROGRESS NOTE^^' then
            begin
              results[i] := '^ PROGRESS NOTE FOLLOWS ';
              scores := False;
            end;
          if Scores then
            result := result + ' ' + piece(results[i], U, 2) + ' = ' +
              piece(results[i], U, 3)
          else
            result := result + results[i] + FILE_NAME_DELIMITER {~};
        end;
    end; // Broker
end;
*)


  (*
function LocationIndexToName(IndexIn: string): string;
begin
  result := '';
  with DataModuleEdit.RPCBroker1 do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ CAPIE';
      Param[0].Mult['"FILEN"'] := '44';
      Param[0].Mult['"FIELDS"'] := '.01';
      Param[0].Mult['"IENS"'] := IndexIn;
      Param[0].PType := List;

      Call;

      // Returns:
      //[DATA]
      //12^Name^2 NORTH

      if not (Results[0] = DATA) then
        Result := BAD_DATA
      else
        Result := Piece(Results[1], U, 3);

    end;
end;  *)


function LoadAnswersForReport(AdministrationID, ADFN: string; Broker: TRPCBroker): string;
// The Answers are in order of the QuestionID.
//    Not in order of appearance on the form: that info is contained in
//    sequence number (second field).

// Output: QuestionID^sequence number^ANSWER
//   Where ANSWER is ChoiceID for Multi-choice items, text for others
var
  TempStr, NewQID, OldQID     : string;
  i,j                         : integer;
  AAnswer                     : TAnswer;
  //question sequece
  str1,str2,temp              : string;
  tempArr                     : array of string;

begin
  Result := OK;
  TempStr := '';

  with Broker do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: AD=' + AdministrationID);
      Remoteprocedure := 'YTQ ALL ANSWERS';
      Param[0].Mult['"AD"'] := AdministrationID;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      // Output: QuestionID^sequence number^ANSWER
      // Where ANSWER is ChoiceID for Multi-choice items, text for others

      OldQID := '';

      // question sequence llh
      if Results.Count > 2 then
      begin
        setLength(tempArr,(results.Count - 2)) ;
        i := 0;
        for j := 2 to Results.Count - 1 do
        begin
          tempArr[i] := Results[j];
          i := i + 1;
        end;

        // patch 129, discovered during gui redesign for re-release
        //bug fix, exposed with scoring issue between question 16
        // and 17 in the BAM-R. loop quits before sorting all out of
        // sequence questions
        for i := 0 to High(tempArr) - 1  do
        begin
          //for j := 0 to (High(tempArr) - 1 ) - i do
          for j := 0 to (High(tempArr) - 1 ) - 1 do
          begin
            //if ( j + 1 = High(tempArr) ) then  continue;
            str1 := Piece(Piece(tempArr[j], U, 2), ';', 1);
            str2 := Piece(Piece(tempArr[j+1], U, 2), ';', 1);
            if strtoint(str1) > strtoint(str2) then
            begin
                temp := tempArr[j];           // temp = old value
                tempArr[j] := tempArr[j+1];   // tempArr[j] = new value
                tempArr[j+1] := temp;         // tempArr[j+1] = old value
            end;
          end;
        end;
      end;
     // end question sequence fix


      if Results.Count > 2 then
        begin
          //for j := 2 to Results.Count - 1 do
          // changed all Results[j] to tempArr[j] in this for loop
          for j := 0 to High(tempArr) do
            begin
              NewQID := Piece(tempArr[j], U, 1);

              // Long text answer
              if (NewQID = OldQID) then
                begin
                  AAnswer.AnswerText := AAnswer.AnswerText + Piece(tempArr[j], U, 3);
                end
              else
                begin
                  AAnswer := TAnswer.Create;
                  AAnswer.Clear;
                  AAnswer.QuestionID := NewQID;
                  AAnswer.AnswerValue := Piece(tempArr[j], U, 3); // ChoiceID
                  AAnswer.AnswerChoiceID := Piece(tempArr[j], U, 3); // ChoiceID
                  AAnswer.AnswerText := Piece(tempArr[j], U, 3);
                  MHA_Report_OPS.ListOfAnswersForReport.Add(AAnswer);
                  OldQID := NewQID;
                end; //else

            end; //for j ...
          //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Successful');
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Error');
          //* for j := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[j]);
          Result := BAD_DATA;
        end;

    end; //with DataModule1.RPCBroker1
end;

{  not used - commented out 07/23/2012
function LoadAnswersForSummary(AdministrationID: string; Broker: TRPCBroker): string;
var
  j                           : integer;
begin
  Result := '';

  with Broker do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: AD=' + AdministrationID);
      Remoteprocedure := 'YTQ ALL ANSWERS';
      Param[0].Mult['"AD"'] := AdministrationID;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results.Count > 2 then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Successful');
          for j := 2 to Results.Count - 1 do
            Result := Result + Piece(Results[j], U, 3) + U;
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ALL ANSWERS: Error');
          //* for j := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[j]);
          Result := BAD_DATA;
        end;
    end; //with DataModule1.RPCBroker1
end;
}

function LoadQuestionsForReport(aInstrument: string; Broker: TRPCBroker): string;
// This an appreviated version contained in uMHA_Question_IO
var
  TempStr                     : string;
  i, x                        : integer;
  AQuestion                   : TQuestion;
  TempList                    : TStringList;

begin
  Result := OK;

  with Broker, Question_OPS do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ QUESTALL: CODE=' + aInstrument);
      RemoteProcedure := 'YTQ QUESTALL';
      Param[0].Mult['"CODE"'] := aInstrument;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ QUESTALL: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ QUESTALL: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      TempList := TStringList.Create;

      for i := 2 to Results.Count - 1 do
        TempList.Add(Results[i]);

      for i := 0 to TempList.Count - 1 do
        begin

          if Pos('QUESTION NUMBER', TempList[i]) > 0 then
            begin

              AQuestion := TQuestion.Create;
              AQuestion.Clear;

              TempStr := Piece(TempList[i], EQ, 2);
              AQuestion.QuestionID := Piece(TempStr, U, 1);
              AQuestion.qTag := StrToIntDef(AQuestion.QuestionID, -1);
              AQuestion.Designator := Piece(TempStr, U, 3);
            end;

          if Pos('QUESTION TEXT', TempList[i]) > 0 then
            begin

              AQuestion.qText := Piece(TempList[i], EQ, 2);

              // The question text can take several lines
              x := 1;
              while (Pos('INTRO TEXT', TempList[i + x]) = 0) and (x < 10) do
                begin
                  AQuestion.qText := AQuestion.qText + ' ' + TempList[i + x];
                  Inc(x);
                end;

              //Set caption
              if (AQuestion.Designator = '') then
                AQuestion.qCaption := AQuestion.qText
              else
                begin
                  if StrToIntDef(AQuestion.Designator, -1) = -1 then
                    AQuestion.qCaption := AQuestion.Designator + ' ' + AQuestion.qText
                  else
                    AQuestion.qCaption := AQuestion.Designator + '. ' + AQuestion.qText;
                end;
            end;

          if Pos('DESC', TempList[i]) > 0 then
          // DESC1= Response Type^CHOICETYPE^MIN^MAX^IS REQUIRED^HINT^CHOICEIDENTIFIER
            begin
              TempStr := Piece(TempList[i], EQ, 2);
              AQuestion.ResponseType := ConvertStringToResponseType(Piece(TempStr, U, 1));
              AQuestion.ChoiceTypeID := Piece(TempStr, U, 2);
              AQuestion.ChoiceIdentifier := ConvertStringToChoiceIdentifier(Piece(TempStr, U, 7));
            end;

          // This is the last statement for this question object
          if Pos('CDISPLAY', TempList[i]) > 0 then
          //  CDISPLAY1=DisplayId^FontName^FontBold^FontItalic
          //  ^FontUnderlined ^Fontsize^FontColor^Alignment
          //  ^left^Mask^Colums^Components
            MHA_Report_OPS.ListOfQuestionsForReport.Add(AQuestion);
        end;

      TempList.Free;
    end; // Broker...

  //* LocalFiles.UpdateAPILog('Call to YTQ QUESTALL: Successful');
  Result := OK;
end;


function LoadQuestionChoicesForReport(aInstrument: string; Broker: TRPCBroker): string;
// This adds the multiple choice items to each question in the ListOfQuestions,
// although some questions will not need this service.
var
  i, j                        : Integer;
  TempStr                     : string;
  TempList                    : TStringList;

begin
  Result := OK;
  TempList := TStringList.Create;

  PiecesToList(ReadChoiceList(aInstrument, Broker), RECORD_END, TempList);

// Raw data from Vista looks like:
// 469^1^100098^One^1
// 469^2^100099^Two^2
// 469^3^100100^Three^3
// 469^4^100101^Four^4


  if (TempList[0] = BAD_DATA) then
    begin
      ShowMessage('Unable to load question choices for ' + aInstrument + '.');
      Result := BAD_DATA;
      TempList.Free;
      Exit;
    end;

  for i := 0 to MHA_Report_OPS.ListOfQuestionsForReport.Count - 1 do
    begin
      for j := 0 to TempList.count - 1 do
        begin
          if TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).ChoiceTypeID =
            Piece(TempList[j], U, 1) then
            begin

              TempStr := Piece(TempList[j], U, 2) + U; // Designator/Sequence
              TempStr := TempStr + Piece(TempList[j], U, 3) + U; // ChoiceID
              TempStr := TempStr + Piece(TempList[j], U, 4) + U; // Text
              TempStr := TempStr + Piece(TempList[j], U, 5); // Legacy Value
//if i = 20 then
// ShowMessage(tempstr);
              TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).MultiChoice.Choices.Add(TempStr);
              TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).MultiChoice.NumberOfPicks :=
                TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).qMax;
              TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).MultiChoice.NumberOfChoices :=
                TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).MultiChoice.NumberOfChoices + 1;
            end;

        end; // for j

    end; // for i

  TempList.Free;

end;


function LoadScalesForReport(aInstrument: string; Broker: TRPCBroker): string;
var
  SubStr, SubSubStr           : string;
  i                           : Integer;
  AScale                      : TScale;
begin
  Result := OK;

  with Broker do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ GET SCALES: CODE=' + aInstrument);
      RemoteProcedure := 'YTQ GET SCALES';
      Param[0].Mult['"CODE"'] := aInstrument;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GET SCALES: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GET SCALES: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          //* LocalFiles.UpdateAPILog('Call to YTQ GET SCALES: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          Exit;
        end
      else
        //* LocalFiles.UpdateAPILog('Call to YTQ GET SCALES: Successful');

  // Usually there will be one scale, but check for more than one.
  // All we really need here is the Scale.IEN and the label.

        if Results.Count > 1 then
          for i := 2 to Results.Count - 1 do
            begin
              SubStr := Results[i];

              if Pos('Scale', SubStr) > 0 then
                begin
                  AScale := TScale.Create;
                  SubSubStr := Piece(SubStr, EQ, 2);
                  AScale.IEN := Piece(SubSubStr, U, 1);
                // AScale.GroupIEN := Piece(SubSubStr, U, 2);
                  AScale.Sequence := StrToIntDef(Piece(SubSubStr, U, 3), -1);
                  AScale.ScaleName := Piece(SubSubStr, U, 4);
                  AScale.XLabel := Piece(SubSubStr, U, 5);

                  MHA_Report_OPS.ListOfScalesForReport.Add(AScale);
                end;
            end;
    end;
end;

function ReadAllKeysInVista(aInstrument: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  Result := BAD_DATA;

  with Broker do
    begin


      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ KEYS: CODE=' + aInstrument);
      RemoteProcedure := 'YTQ KEYS';
      Param[0].Mult['"CODE"'] := aInstrument;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ KEYS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ KEYS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] = DATA then
        begin
          for i := 1 to Results.Count - 1 do
            Result := Result + Results[i] + RECORD_END;
          //* LocalFiles.UpdateAPILog('Call to YTQ KEYS: Successful');
          Result := DropTrailer(Result);
        end;

      // No keys for this instrument
      if Results[0] = '[ERROR]' then
        if Results.Count > 0 then
          if Results[1] = 'no scale grps found' then
            Result := '';
          //* else
            //* LocalFiles.UpdateAPILog('Call to YTQ KEYS: Error ' + Results[1]);

    end;
end;


function ReadScoringKeys(aInstrument: string; Broker: TRPCBroker): boolean;
var
  tmpAllKeys,
    tmpRecord,
    tmpKey,
    tmpScaleID
                              : string;
  i                           : integer;
  AKey                        : TKey;

  //[DATA]
  //SCALE=Score^100088^
  //KEY=3821^Some or a little of the time (1-2 days)^1^100010
  //KEY=3821^Occasionally or a moderate amount of time (3-4 days)^2^100011
  //KEY=3821^Most or all of the time (5-7 days)^3^100012

begin
  Result := True;

  tmpAllKeys := ReadAllKeysInVista(aInstrument, Broker);

  // No keys for this instrument
  if (tmpAllKeys = '') then
    Exit;

  if (tmpAllKeys = BAD_DATA) then
    begin
      Result := False;
      Exit;
    end;

  tmpRecord := Piece(tmpAllKeys, RECORD_END, 1);
  tmpScaleID := Piece(Piece(tmpRecord, EQ, 2), U, 2);

  for i := 2 to NumberOfPieces(tmpAllKeys, RECORD_END) do
    begin
      tmpRecord := Piece(tmpAllKeys, RECORD_END, i);

      if (pos('SCALE', tmpRecord) > 0) then
        begin
          tmpScaleID := Piece(Piece(tmpRecord, EQ, 2), U, 2);
        end //is a new scale
      else
        begin
          AKey := TKey.Create;

          tmpKey := Piece(tmpRecord, EQ, 2);
          AKey.ScaleID := tmpScaleID;
          AKey.QuestionID := Piece(tmpKey, U, 1);
          AKey.AnswerText := Piece(tmpKey, U, 2);
          AKey.Value := Piece(tmpKey, U, 3);
          //patch 129 llh - need key ien

          AKey.IEN := Piece(tmpKey,U,4) ;

          MHA_Report_OPS.ListOfKeysForReport.Add(AKey);
        end;
    end; // for i
end;

//patch 123 -- load RESULTS from a specific administration
function LoadAdminResults(AdministrationID: string; Broker: TRPCBroker) : string;
var
  i                           : integer;
  //TempStrList                 : TStringList;
begin
  Result := '' ;

  if (AdministrationID = '') then
    exit   ;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ GET RESULTS';

       //* LocalFiles.UpdateAPILog('Call to YTQ GET RESULTS: ' +
       //*   ' AD=' + AAdministrationIENS);
      Param[0].Mult['"AD"'] := AdministrationID;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GET RESULTS Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GET RESULTS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      // This will return '[ERROR]' when there are no scales, or scores, for this
      // instrument: Not really an error.
      if (Results[0] <> DATA) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ SCORE ADMIN: Error ' + Results[0]);
          Result := '';
        end
      else
        begin
          // Format is: ScaleName=ScaleScore
          for i := 1 to Results.Count - 1 do
            Result := Result + Results[i] + RECORD_END;


          // llh --- check, may not want to drop the trailer,
          Result := DropTrailer(Result);
          //* LocalFiles.UpdateAPILog('Call to YTQ GET RESULTS: Successful');
        end;
    end;
end;



(*function GetNotesList(DFN: string; Broker: TRPCBroker): TStrings;
begin

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('OR CPRS GUI CHART');

      RemoteProcedure := 'TIU NOTES';
      Param[0].Value := DFN;
      Param[0].PType := literal;

      result := results;
    end;
end; *)

function GetPatientInfo(DFN: string; Broker: TRPCBroker): TStrings;
begin
  result.Clear;

  with Broker do
    begin

      CreateContext('OR CPRS GUI CHART');
      ClearParameters := True;
      ClearResults := True;
      //* LocalFiles.UpdateAPILog('Call to ORWPT PTINQ: DFN=' + DFN);
      RemoteProcedure := 'ORWPT PTINQ';
      Param[0].Value := DFN;
      Param[0].PType := literal;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to ORWPT PTINQ: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to ORWPT PTINQ: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      result := results;
      //* LocalFiles.UpdateAPILog('Call to ORWPT PTINQ: Successful');
    end;
end;


end.

