
unit uMHA_Rule_OPS;

interface

uses
  classes,
  fIA_dialogs, Dialogs,
  uIA_Rule,
  uMHA_Administration,
  uIA_Instrument,
  DateUtils,
  SysUtils,
  System.Contnrs;


type

  TRule_OPS = class
  private
    FSkipQuestions: string;
    function TestRule(IsFirstCondition: boolean;
      ARule: TRule; AnswerStr: string): Boolean;
  protected
  public
    destructor Destroy; override;
    constructor Create;
    property SkipQuestions: string read FSkipQuestions write FSkipQuestions;

    function ProcessRule(aAdmin: TAdministration;
      aInstrument: TInstrument; QID, AnswerStr: string): string;
    //function RuleStatement(QID: string): string;
    //procedure GetSkipQuestionsByRule(ARule: TRule);
    //procedure RemoveSkipQuestionsByRule(ARule: TRule);
    procedure RemoveSkipQuestionsByRule(Questions: string);

    //patch 108 llh - custom rule/skip question logic for the FOCI instrument
    procedure TestFOCISkipQues(aAdmin: TAdministration;
      aInstrument: TInstrument; QID, AnswerStr: string);

  end; { TRule_OPS }

var
  Rule_OPS                    : TRule_OPS;

implementation

uses
  uIA_Const,
  uIA_Utils,
  uIA_Question,
  uIA_Answer,
  uIA_Rule_IO;

var
  lvQuestion                  : TQuestion;


constructor TRule_OPS.Create;
begin
  inherited Create;
  lvQuestion := TQuestion.Create;
  FSkipQuestions := '';
  //SkipQuestions := ReadQuestionsToSkip(Instrument.InstrumentName);
end;

destructor TRule_OPS.Destroy;
begin
  lvQuestion.Free;
  inherited Destroy;
end;


function TRule_OPS.TestRule(IsFirstCondition: boolean;
  ARule: TRule; AnswerStr: string): Boolean;

var
  tmpValueDataType            : TValueDataType;
  tmpValue                    : string;
  tmpOperator                 : TOperatorType;
  tmpInterval                 : integer;

  function MyStrToOperator(AOperatorString: string): TOperatorType;
  var
    tmpIndex                  : integer;
    tmpString                 : string;
  begin
    //  toEQ, toLT, toLE, toGT, toGE, toNE
    //  OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
    //    + 'Equals or is less than^Equals or is greater than';

    tmpString := AOperatorString;
    tmpIndex := PieceIndex(OPERATOR_STRINGS, tmpString, U);

    case tmpIndex of
      2: Result := toNE;
      3: Result := toLT;
      4: Result := toGT;
      5: Result := toLE;
      6: Result := toGE;
    else
      Result := toEQ;
    end; //case

  end; //MyStrToSecondOp

  function MyStrToDataType(ADataTypeString: string): TValueDataType;
  begin
    //dtString, dtDate, dtNumber
    Result := dtString;
    if (ADataTypeString = DATE_STR) then
      Result := dtDate;
    if (ADataTypeString = NUMBER_STR) then
      Result := dtNumber;
    if (ADataTypeString = INTERVAL_STR) then
      Result := dtTimeInterval;
  end; //MyStrToDataType

begin

  Result := False;
  tmpInterval := 0;

  if IsFirstCondition then
    begin
      tmpValueDataType := ARule.FirstValueDataType;
      tmpValue := ARule.FirstValue;
      tmpOperator := ARule.FirstOperator;
    end
  else
    begin
      tmpValueDataType := ARule.SecondValueDataType;
      tmpValue := ARule.SecondValue;
      tmpOperator := ARule.SecondOperator;
    end;

  if (tmpValueDataType = dtTimeInterval) then
    begin
      tmpInterval := StrToIntDef(Piece(tmpValue, '*', 1), 0);
      tmpValue := Piece(tmpValue, '*', 2);
      if (tmpValue = 'YEAR') then
        tmpInterval := tmpInterval * 365;
      if (tmpValue = 'MONTH') then
        tmpInterval := tmpInterval * 30;
      if (tmpValue = 'WEEK') then
        tmpInterval := tmpInterval * 7;
      if (tmpValue = 'DAY') then
        tmpInterval := tmpInterval * 1;
    end; //if (tmpValueDataType...

  case tmpOperator of
    toEQ:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr = tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) = StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) = StrToFloat(tmpValue));
          dtTimeInterval:
            Result := (DaysBetween(StrToDate(AnswerStr), Now) = tmpInterval)
        end; //case tmpValueDataType
      end;
    //if (tmpValueDataType = dtTimeInterval) then
    //  Result := (DaysApart(StrToDate(AnswerStr), Now) = tmpInterval)
    //else
    //  Result := (AnswerStr = tmpValue);
    toLT:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr < tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) < StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) < StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) < tmpInterval);
        end; //case tmpValueDataType
      end; //toLT
    toLE:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr <= tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) <= StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) <= StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) <= tmpInterval);
        end; //case tmpValueDataType
      end; //toLE
    toGT:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr > tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) > StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) > StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) > tmpInterval);
        end; //case tmpValueDataType
      end; //toGT
    toGE:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr >= tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) >= StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) >= StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) >= tmpInterval);
        end; //case tmpValueDataType
      end; //toGE
    toNE:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr <> tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) <> StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) <> StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) <> tmpInterval);
        end; //case tmpValueDataType
      end; //toNE
  end; //case tmpOperator

end;


(*procedure TRule_OPS.GetSkipQuestionsByRule(ARule: TRule);
begin
  FSkipQuestions := FSkipQuestions + U + ARule.SkipQuestionIDs;
end;  *)


(*procedure TRule_OPS.RemoveSkipQuestionsByRule(ARule: TRule);
var
  TempStr                     : string;
  i, x                        : integer;
begin

  TempStr := ARule.SkipQuestionIDs;

  if TempStr = '' then
    exit;
  TempStr := TempStr + U;

  for i := 1 to NumberOfPieces(TempStr, U) do
    begin
      // Place delimiter on each side of QuestionID to ensure that the whole
      // number is evaluated.
      x := Pos(U + Piece(TempStr, U, i) + U, FSkipQuestions);
      if x > 0 then
        Delete(FSkipQuestions, x, Length(Piece(TempStr, U, i)) + 1);
    end;
end; *)



procedure TRule_OPS.RemoveSkipQuestionsByRule(Questions: string);
var
  i, j                        : integer;
begin
  for i := NumberOfPieces(FSkipQuestions, U) downto 1 do
    for j := NumberOfPieces(Questions, U) downto 1 do
      begin
        if Piece(Questions, U, j) = Piece(FSkipQuestions, U, i) then
          SetPiece(FSkipQuestions, U, i, ' ');
      end;
end;

function TRule_OPS.ProcessRule(aAdmin: TAdministration;
  aInstrument: TInstrument;   QID, AnswerStr: string): string;
// This will:
// 1. Check for Second Condition: if true, check the first condition and if it is
//    true then present the message and/or update list of questions to skip.
// 2. When no Second Condition, check the First Condition and if true then
//    present message and/or update list of questions to skip.
// The out-going string is the message, if one exists, when the rule is true.

var
  i                           : Integer;
  First, Second               : boolean;
  TempSkips                   : string;

  function GetAnswer(QID: string): string;
  var
    j                         : Integer;
  begin
    Result := '';
    for j := 0 to aAdmin.ListOfAdminAnswers.Count - 1 do
      begin
        if QID = TAnswer(aAdmin.ListOfAdminAnswers[j]).QuestionID then
          begin
            // CheckList is not considered multiple choice here
            if TAnswer(aAdmin.ListOfAdminAnswers[j]).IsMultiChoice then
              Result := TAnswer(aAdmin.ListOfAdminAnswers[j]).AnswerValue
            else
              Result := TAnswer(aAdmin.ListOfAdminAnswers[j]).AnswerText;
            Exit;
          end;
      end;
  end;

begin
  Result := '';
  TempSkips := '';

  if aInstrument.ListOfRules.Count = 0 then
    Exit;

  //patch 108 llh -- if instrument is the FOCI, do other logic to test for
  //                 skipped questions.
  if aInstrument.InstrumentName = 'FOCI' then
  begin
    TestFOCISkipQues(aAdmin, aInstrument, QID, AnswerStr);
    Exit ;
  end;
  // end FOCI test block;

  for i := 0 to aInstrument.ListOfRules.Count - 1 do
    begin
      First := false;
      Second := False;

      // Check for first condition when no second condition exists.
      if (QID = TRule(aInstrument.ListOfRules[i]).FirstQuestionID) then
        begin
          if (TRule(aInstrument.ListOfRules[i]).SecondQuestionID = '') then
            begin
              First := TestRule(True {1st Condition},
                TRule(aInstrument.ListOfRules[i]), AnswerStr);
              if First then
                begin
                  Result := TRule(aInstrument.ListOfRules[i]).MessageText;
                  TempSkips := TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs;
                end
              else
                begin
                  if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                    RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);
                end;
            end; // 1st question only

          // Check First Question, and then the second question
          if (TRule(aInstrument.ListOfRules[i]).SecondQuestionID <> '') then
            begin
              First := TestRule(True {1st Condition},
                TRule(aInstrument.ListOfRules[i]), AnswerStr);

              if First then
                begin
                  Second := TestRule(False {2nd Condition},
                    TRule(aInstrument.ListOfRules[i]),
                    GetAnswer(TRule(aInstrument.ListOfRules[i]).SecondQuestionID));
                  if not Second then
                    if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                      RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);
                end
              else // Not First
                if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                  RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);

              if First and Second then
                begin
                  Result := TRule(aInstrument.ListOfRules[i]).MessageText;
                  TempSkips := TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs;
                end;
            end;
        end; // First Question is current

      // Second Question is current
      if QID = TRule(aInstrument.ListOfRules[i]).SecondQuestionID then
        begin
          Second := TestRule(False {2nd Condition},
            TRule(aInstrument.ListOfRules[i]), AnswerStr);
          if Second then
            begin
              First := TestRule(True {1st Condition},
                TRule(aInstrument.ListOfRules[i]),
                GetAnswer(TRule(aInstrument.ListOfRules[i]).FirstQuestionID));
              if not First then
                if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                  RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);
            end
          else // not second
            if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
              RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);

          if First and Second then
            begin
              Result := TRule(aInstrument.ListOfRules[i]).MessageText;
              TempSkips := TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs;
            end;
        end; // QID = SecondQuestionID
    end; // for i

  // Each QuestionID must be bracketed by a delimiter; this ensures the ID
  // is read properly.
  FSkipQuestions := FSkipQuestions + U + TempSkips + U;

end;


// patch 108 llh -- skip logic for the FOCI is more complex than editor can
//               handle, do not want to remove the skipquestionids as all
//               20 questions use the same logic
procedure TRule_OPS.TestFOCISkipQues(aAdmin: TAdministration;
      aInstrument: TInstrument; QID, AnswerStr: string);
var
  i                         : integer ;
  clearSkips                : boolean ;

begin
  // yes this is bad, BUT can be done because all the rule logic is the
  // same for the skipped questions and is specific to the FOCI instrument.

  // DO NOT CHANGE RULE LOGIC IN THE FOCI UNLESS THIS CODE IS RE-EVALUATED

  //initialize
  clearSkips := false ;
  FSkipQuestions := TRule(aInstrument.ListOfRules[1]).SkipQuestionIDs + U;

  for i := 0 to aInstrument.ListOfQuestions.Count - 6 do    // -6 don't need to evaluate last 5 questions
  begin
    {
    showmessage('i:  = ' + inttostr(i) + #13 +
                'Question ID: ' + TQuestion(aInstrument.ListOfQuestions[i]).QuestionID + #13 +
                'answerText:  ' + TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerText + #13 +
                'AnswerValue: ' + TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerValue + #13 +
                'AnswerStr:  ' + AnswerStr ) ;


    if (QID = TQuestion(aInstrument.ListOfQuestions[i]).QuestionID) AND (AnswerStr = '0') then
      clearSkips := true;

    if ((TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerText = 'Yes') and
        (AnswerStr <> '1')) then //or
    //    (QID = TQuestion(aInstrument.ListOfQuestions[i]).QuestionID) AND (AnswerStr = '0')) then
      clearSkips := True
    }

    if ( (QID = TQuestion(aInstrument.ListOfQuestions[i]).QuestionID)) then
      begin
        if (AnswerStr = '0') or (AnswerStr = '241') then clearskips := true ;
        //else clearskips := false ;
        //if clearskips then break;
      end
    else
      if (AnswerStr = '0') or (TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerText = '1. Yes' ) or
         (TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerText = 'Yes' ) or
         (TAnswer(aAdmin.ListOfAdminAnswers[i]).AnswerText = '241' ) then
        clearskips := true ;
      //else
      //  clearskips := false ;
  end;

  if clearSkips then
    RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[1]).SkipQuestionIDs)  ;
end;

 (*
function TRule_OPS.ProcessRule(aAdmin: TAdministration;
  aInstrument: TInstrument;
  QID, AnswerStr: string): string;
// QID is the QuestionID of the current question; AnswerStr is the ItemIndex for
//  radio groups.
// This will:
// 1. Check for Second Condition: if true, check the first condition and if it is
//    true then present the message and/or update list of questions to skip.
// 2. When no Second Condition, check the First Condition and if true then
//    present message and/or update list of questions to skip.
// The out-going string is the message, if one exists, when the rule is true.

var
  i                           : integer;
  TempSkips,
  TempAnswer                  : string;

  function GetFirstConditionAnswer(QID: string): string;
  var
    j                         : Integer;
  begin
    Result := '';
    for j := 0 to aAdmin.ListOfAnswers.Count - 1 do
      begin
        if QID = TAnswer(aAdmin.ListOfAnswers[i]).QuestionID then
          begin
            Result := TAnswer(aAdmin.
            ).AnswerText;
            Exit;
          end;
      end;
  end;

begin
  Result := ''; // Message if rule is triggered
   TempSkips := '';

  // No rules, no message, no questions to skip, stop here
  if aInstrument.ListOfRules.Count = 0 then
    begin
      FSkipQuestions := '';
      Exit;
    end;

  for i := 0 to aInstrument.ListOfRules.Count - 1 do
    begin
      // QID is the current question's ID
      // Begin by checking for second condition
      if QID = TRule(aInstrument.ListOfRules[i]).SecondQuestionID then
        begin
          if TestRule(False {2nd Condition},
            TRule(aInstrument.ListOfRules[i]),
            AnswerStr) then
            begin
              // 2nd Condition true; Load the 1st Condition Question
              TempAnswer := GetFirstConditionAnswer(TRule(aInstrument.ListOfRules[i]).FirstQuestionID);
              //TempAnswer := frmMHA_DataEntry.ReadAnswer(TRule(Instrument.ListOfRules[i]).FirstQuestionID);
              if TestRule(True {1st Condition},
                TRule(aInstrument.ListOfRules[i]),
                TempAnswer) then
                begin
                  Result := TRule(aInstrument.ListOfRules[i]).MessageText;
                  GetSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]));
                  Exit;
                end;
            end
          else
            RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]));
        end; // QID = Second condition

      // Check for first condition when no second condition exists
      if (QID = TRule(aInstrument.ListOfRules[i]).FirstQuestionID) and
        (TRule(aInstrument.ListOfRules[i]).SecondQuestionID = '') then
        begin
          TempAnswer := AnswerStr;
          if TestRule(True {1st Condition},
            TRule(aInstrument.ListOfRules[i]),
            TempAnswer) then
            begin
              Result := TRule(aInstrument.ListOfRules[i]).MessageText;
              GetSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]));
              exit;
            end
          else
            RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]));
        end;
    end;
end; *)

initialization
  Rule_OPS := TRule_OPS.Create;

finalization
  Rule_OPS.Free;


end.

