unit fMHA_DLL_GAF;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Math,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  StdCtrls,
  Grids,
  BaseGrid,
  AdvGrid,
  RzPanel,
  TeeProcs,
  TeEngine,
  Chart,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  ComCtrls,
  Spin,
  Series,
  Mask,
  JvExMask,
  JvSpin, AdvObj;

type
  TfrmMHA_DLL_GAF = class(TForm)
    pnlNewGAF: TPanel;
    pnlOldGAF: TPanel;
    pnlCriteria: TPanel;
    Splitter1: TSplitter;
    pnlOldGAFLeft: TPanel;
    Splitter2: TSplitter;
    pnlOldGAFRight: TPanel;
    grpOldGAFs: TRzGroupBox;
    grdGAFResults: TAdvStringGrid;
    grpCriteria: TRzGroupBox;
    grdCriteria: TAdvStringGrid;
    chrtGAF: TChart;
    lblEnterGAF: TLabel;
    btnFinished: TJvXPButton;
    Series1: TLineSeries;
    btnCancel: TJvXPButton;
    spnedtGAF: TJvSpinEdit;

    procedure FormShow(Sender: TObject);

    procedure spnedtGAFChange(Sender: TObject);
    procedure grdGAFCriteriaClick(Sender: TObject);

    procedure FormCanResize(Sender: TObject; var NewWidth,
      NewHeight: Integer; var Resize: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);

    procedure grdGAFCriteriaGridHint(Sender: TObject; ARow, ACol: Integer;
      var hintstr: string);
    procedure FormCreate(Sender: TObject);
    procedure btnFinishedClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure spnedtGAFKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure grdGAFResultsGetCellColor(Sender: TObject; ARow,
      ACol: Integer; AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

  private
    FspnGAFChange: BOOLEAN; //local variable used in spnGAF and grdGAFResults
    procedure InitGAFCriteria;
    procedure InitGrdResults;
    procedure InitForm;
    procedure DisplayPreviousRatings;
    procedure GraphRatings;
    procedure UpdateAllDisplays;
    function RowTag(ARow: integer): string;
    procedure LoadGAFByRow(ARow: integer);
  public
    { Public declarations }
  end;

var
  frmMHA_DLL_GAF              : TfrmMHA_DLL_GAF;

implementation

{$R *.dfm}

uses
  uIA_Const,
  uMHA_DLL_Const,
  uMHA_DLL_Registry,
  uIA_Utils,
  fIA_Dialogs,
  uIA_Patient,
  uMHA_DLL_GAF,
  uMHA_DLL_GAF_List,
  uIA_GUIControl;

const
  GAFCopyright                : string = '  Reprinted with permission from the '
    + 'Diagnostic and Statistical Manual of Mental Disorders, Fourth Edition.'
    + #13 + '  Copyright 1994 American Psychiatric Association.';
var
  SaveTheScore                : Boolean;


procedure TfrmMHA_DLL_GAF.FormCreate(Sender: TObject);
var
  CPRSFont                    : HFONT;
begin
// The screen dimensions are set to the user's last setting.
  //gvFont := TFont.Create;

  MHA_DLL_Registry.ReadRegistry;
  Top := gvTop;
  Left := gvLeft;
  Height := gvHeight;
  Width := gvWidth;
  gvUseDesignFont := True;

        // Look for CPRS main window's class
  gvCPRS_Instance_Handle := FindWindow('TfrmFrame', nil); //CPRS
  if gvCPRS_Instance_Handle <> 0 then // Window found
    begin
      CPRSFont := SendMessage(gvCPRS_Instance_Handle, WM_GETFONT, 0, 0);
      gvFont := MakeFontFromHFont(CPRSFont);
      //MHA_DLL_Registry.SaveRegistry;
    end;

  Font.Assign(gvFont);

  ShowModal;
end;


procedure TfrmMHA_DLL_GAF.FormShow(Sender: TObject);
begin

  Caption := '  Global Assessment of Functioning for ' + Patient.Name + '  ';

  if gvUseScreenReader then
    begin
      Caption := 'G A F for patient ' + Patient.Name;
    // Don't call these a "table." JAWS has special keys for reading tables
    // which will not work here.
      grpOldGAFs.Caption := 'The following grid shows G A F ratings. The' +
        ' three columns show the date, G A F rating and the clinician. Navigate' +
        ' using the arrow keys. Press tab to enter grid.';
      grpCriteria.Caption :=
        'The next grid summarizes G A F criteria from low to ' +
        'high scores. Navigate with the arrow keys. Press tab to enter grid.';
      grpOldGAFs.Width := Width - 10;
      grpOldGAFS.TabStop := True;
      grpCriteria.TabStop := True;

      ActiveControl := grpOldGAFs;
    end;

  InitForm;
  InitGAFCriteria;
  InitGrdResults;
  FinishedAs := faCancelled;

  UpdateAllDisplays;

  gvScoreStr := '0'; // + U + DateToStr(DtpkrEval.date);
  spnedtGAF.Value := 0;
  btnFinished.Enabled := False;
  SaveTheScore := False;
end;

procedure TfrmMHA_DLL_GAF.InitForm;
begin
  with GUI_Control do
    begin
      GUI_Control.ConfigureGUI(gvFont);
      font.Assign(gvFont);
      btnFinished.Font.Assign(gvFont);
      pnlNewGAF.Height := 2 * ButtonHeight;
      btnFinished.height := ButtonHeight;
      btnCancel.Height := ButtonHeight;

      lblEnterGAF.height := 3 * TextHeight('Yy', gvFont) div 2;
      spnedtGAF.height := 3 * TextHeight('Yy', gvFont) div 2;
      btnFinished.top := ButtonHeight div 2;
      btnCancel.top := ButtonHeight div 2;
      lblEnterGAF.top := (pnlNewGAF.Height div 2) - (lblEnterGAF.height div 2);
      spnedtGAF.top := (pnlNewGAF.Height div 2) - (spnedtGAF.height div 2);

      btnFinished.width := ButtonWidth(btnFinished.caption);
      btnCancel.width := btnFinished.Width;
      lblEnterGAF.width := TextWidth(lblEnterGAF.Caption, gvFont);
      spnedtGAF.width := TextWidth('XXXXXXX', gvFont);

      lblEnterGAF.Left := pnlNewGAF.Left + 10;
      spnedtGAF.Left := lblEnterGAF.Left + lblEnterGAF.Width + 5;
      btnFinished.Left := spnedtGAF.Left + spnedtGAF.Width + 20; //dtpkrEval.Left + dtpkrEval.Width + 20;
      btnCancel.Left := btnFinished.Left + btnFinished.Width + 20;
    end;
end;


procedure TfrmMHA_DLL_GAF.InitGrdResults;
begin
  //init string grid
  with grdGafResults, GUI_Control do
    begin
      Font.Assign(gvFont);

      if gvUseScreenReader then
        begin
          SelectionRectangle := True;
          ShowSelection := True;
          Options := [goFixedVertLine, goFixedHorzLine, goHorzLine,
            goDrawFocusSelected, goColSizing, goRowSelect];
          FixedRowAlways := False;
          FixedColAlways := False;
          FixedRows := 0;
          FixedCols := 0;
          ShowHint := False;
        end
      else
        begin
          grpOldGAFs.Caption := ' ';
          SelectionRectangle := False;
          ShowSelection := True;
          Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine,
            goDrawFocusSelected, goColSizing, goRowSelect];
          FixedFont.Assign(gvFont);
          FixedRowAlways := True;
          FixedColAlways := False;
          FixedRows := 1;
          FixedCols := 0;
          FixedRowHeight := DefaultRowHeight;
          ShowHint := True;
        end;

      DefaultRowHeight := 3 * TextHeight('Yy', gvFont) div 2;
      ColCount := 5;
      ColWidths[0] := TextWidth('XXX/XX/XXXXXXX', gvFont); // Date
      ColWidths[1] := TextWidth('XXXXX', gvFont); //40; //Rating
      ColWidths[2] := TextWidth('XXXXXXXXXXXXXXXXXXXXXXXXXXXXX', gvFont);
    //200; //Rater
      ColWidths[3] := 0; //IENS
      ColWidths[4] := 0; //row tag
      Cells[0, 0] := 'Date';
      Cells[1, 0] := 'GAF';
      Cells[2, 0] := 'Clinician';
    end;

end;

procedure TfrmMHA_DLL_GAF.InitGAFCriteria;
var
  i                           : integer;
begin

  with grdCriteria, GUI_Control do
    begin
      WordWrap := true;
      Font.Assign(gvFont);

      if gvUseScreenReader then
        begin
          SelectionRectangle := False;
          ShowSelection := true;
          Options := [goFixedVertLine, goFixedHorzLine, goHorzLine, goColSizing,
            goRowSelect];
          FixedRowAlways := False;
          FixedColAlways := False;
          FixedRows := 0;
          FixedCols := 0;
          ShowHint := False;
        end
      else
        begin
          FixedFont.Assign(gvFont);
          FixedRowHeight := 3 * TextHeight('Yy', gvFont) div 2;
          SelectionRectangle := false;
          ShowSelection := True;
          Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine,
            goDrawFocusSelected, goColSizing, goRowSelect];
          FixedFont.Assign(gvFont);
          FixedRowAlways := True;
          FixedColAlways := False;
          FixedRows := 1;
          FixedCols := 0;
          FixedRowHeight := DefaultRowHeight;
          ShowHint := True;
        end;

      ColWidths[0] := TextWidth('XXRangeXX', gvFont);
      Cells[0, 0] := 'Range';
      Cells[0, 1] := '1-10';
      Cells[0, 2] := '11-20';
      Cells[0, 3] := '21-30';
      Cells[0, 4] := '31-40';
      Cells[0, 5] := '41-50';
      Cells[0, 6] := '51-60';
      Cells[0, 7] := '61-70';
      Cells[0, 8] := '71-80';
      Cells[0, 9] := '81-90';
      Cells[0, 10] := '91-100';

      Cells[1, 0] := 'Criteria';

      Cells[1, 1] :=
        'Persistent danger of severely hurting self or others (e.g., '
        + 'recurrent violence) OR persistent inability to maintain minimal personal '
        + 'hygiene OR serious suicidal act with clear expectation of death.';

      Cells[1, 2] :=
        'Some danger of hurting self or others (e.g., suicide attempts '
        + 'without clear expectation of death; frequently violent; OR occasionally '
        + 'fails to maintain minimal personal hygiene (e.g., smears feces) OR gross '
        + 'impairment in communication (e.g., largely incoherent or mute).';

      Cells[1, 3] :=
        'Behavior is considerably influenced by delusions or hallucinations OR '
        + 'serious impairment in communication or judgment (e.g., sometime incoherent, '
        + 'acts grossly inappropriately, suicidal preoccupation) OR inability to '
        + 'function in almost all areas (e.g., stays in bed all day; no job, home or friends).';

      Cells[1, 4] :=
        'Some impairment in reality testing or communication (e.g., speech is at '
        + 'times illogical, obscure, or irrelevant) OR major impairment in several '
        + 'areas, such as work or school, family relations, judgment, thinking, or '
        + 'mood (e.g., depressed man avoids friends, neglects family, and is unable '
        + 'to work; child frequently beats up younger children, is defiant at home '
        + 'and is failing at school).';

      Cells[1, 5] :=
        'Serious symptoms (e.g., suicidal ideation, severe obsessional rituals, '
        + 'frequent shoplifting) OR any serious impairment in social, occupational, '
        + 'or school functioning (e.g., no friends, unable to keep a job).';

      Cells[1, 6] :=
        'Moderate symptoms (e.g., flat affect and circumstantial speech, occasional '
        + 'panic attacks) OR moderate difficulty in social, occupational, or school '
        + 'functioning (e.g., few friends, conflicts with peers or co-workers).';

      Cells[1, 7] :=
        'Some mild symptoms (e.g., depressed mood and mild insomnia) OR some '
        + 'difficulty in social, occupational or school functioning (e.g., occasional '
        + 'truancy, or theft within the household), but generally functioning '
        + 'pretty well, has some meaningful interpersonal relationships.';

      Cells[1, 8] :=
        'If symptoms are present, they are transient and expectable reactions to '
        + 'psychosocial stressors (e.g., difficulty concentrating after family '
        + 'argument); no more than slight impairment in social, occupational, or '
        + 'school functioning (e.g., temporarily falling behind in schoolwork).';

      Cells[1, 9] :=
        'Absent or minimal symptoms (e.g., mild anxiety before an exam), good '
        + 'functioning in all areas, interested and involved in a wide range '
        + 'of activities, socially effective, generally satisfied with life, no '
        + 'more than everyday problems or concerns (e.g., an occasional argument '
        + 'with family members).';

      Cells[1, 10] := 'Superior functioning in a wide range of activities, '
        + 'life' + '''' +
        's problems never seem to get out of hand, is sought out '
        + 'by others because of his or her many positive qualitites.  No symptoms.';

      for i := 1 to 10 do
        begin
          RowHeights[i] := (1 + Ceil(((TextWidth(Cells[1, i], gvFont) + ColWidths[1] div 4) /
            (ColWidths[1] - 25 {width of scroll bar}))))
            * (TextHeight('Yy', gvFont));

        end;
    end;

end;

procedure TfrmMHA_DLL_GAF.UpdateAllDisplays;
begin
  DisplayPreviousRatings;

  GAF.Clear;
  FspnGAFChange := False;
  grdGAFResults.Row := 1;
end;

procedure TfrmMHA_DLL_GAF.DisplayPreviousRatings;
var
  i                           : integer;
begin

  with grdGafResults do
    //if there are previous ratings, write them to string grid
    if PatientGAFList.PreviousRatingsExist then
      begin

        grdGafResults.RowCount := PatientGAFList.NumberOfRatings + 1;
        for i := 0 to PatientGAFList.NumberOfRatings - 1 do
          begin
            PatientGAFList.LoadByIndex(i);
            Cells[0, i + 1] := GAF.DateString;
            Cells[1, i + 1] := IntToStr(GAF.Rating);
            Cells[2, i + 1] := GAF.Rater;
            Cells[3, i + 1] := GAF.IENS;

            //Set row tag
            Cells[4, i + 1] := '';
            if GAF.EnteredInError then
              Cells[4, i + 1] := 'ERROR';
//            if GAF.CanBeEdited then
//              Cells[4, i + 1] := 'EDIT';
//            if GAF.CanBeMarkedInError then
//              Cells[4, i + 1] := 'MARK';
          end; //for i...
      end //if previous ratings exist

    else //if there are no previous GAF ratings
      begin
        RowCount := 2;
        Cells[0, 1] := 'None';
        Cells[1, 1] := '';
        Cells[2, 1] := '';
      end; //else

  grdGAFResults.SortByColumn(0);
  grdGAFResults.TopRow := 1;

  GraphRatings;
end;


procedure TfrmMHA_DLL_GAF.spnedtGAFChange(Sender: TObject);
var
  tmpRow                      : Integer;
begin

  //Correct for garbage keyboard entries
  with spnedtGAF do
    begin
      if pos('0', Text) = 1 then
        Text := copy(Text, 2, Length(Text) - 1);
      if Text = '' then
        exit;
      if Value > MaxValue then
        Value := MaxValue;
      if Value < MinValue then
        Value := MinValue;
    end;

  //Reset selected row of all criteria string grid
  FspnGAFChange := True;
  tmpRow := ((Round(spnedtGAF.Value) - 1) div 10) + 1;

  grdCriteria.Row := tmpRow;
  FspnGAFChange := False;
  FinishedAs := faComplete;

  gvScoreStr := IntToStr(Round(spnedtGAF.Value)); // + U + DateToStr(dtpkrEval.date);
  if (Piece(gvScoreStr, U, 1) = '0') or (spnedtGAF.text = '') or (spnedtGAF.text = ' ') then
    begin
      gvScoreStr := '0'; // + U + DateToStr(dtpkrEval.date);
      btnFinished.Enabled := False;
    end
  else
    btnFinished.Enabled := True;
end;

procedure TfrmMHA_DLL_GAF.grdGAFCriteriaClick(Sender: TObject);
var
  LastDigit                   : integer;
begin
  //compute the last digit of the rating
  //otherwise, entering a value via keyboard to spnGAF gets stepped on
  //by this event
  if FspnGAFChange then
    LastDigit := Round(spnedtGAF.Value) mod 10
  else
    LastDigit := 5;

  if LastDigit = 0 then
    LastDigit := 10;

  //reset the rating
  spnedtGAF.Value := ((grdCriteria.Row - 1) * 10) + LastDigit;

end;

procedure TfrmMHA_DLL_GAF.GraphRatings;
var
  irow                        : integer;
begin

  chrtGAF.Visible := PatientGAFList.RatingsCanBeGraphed;

  if not PatientGAFList.RatingsCanBeGraphed then
    exit;

  with chrtGAF do
    begin
      Title.Text.Clear;
      Series1.Clear;

      for irow := grdGafResults.RowCount - 1 downto 1 do
        begin
          LoadGAFByRow(irow);

          if GAF.EnteredInError then
            continue;

          Series1.AddXY(GAF.Date, GAF.Rating, GAF.DateString, clRed);
        end;
    end;

end;


function TfrmMHA_DLL_GAF.RowTag(ARow: integer): string;
begin
  Result := grdGAFResults.Cells[4, ARow];
end;

procedure TfrmMHA_DLL_GAF.LoadGAFByRow(ARow: integer);
begin
  if PatientGAFList.PreviousRatingsExist then
    PatientGAFList.LoadByIENS(grdGAFResults.Cells[3, ARow]); //IENS
end;


procedure TfrmMHA_DLL_GAF.FormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
  Resize := true;
  if NewWidth < 770 then
    Resize := false;
end;

procedure TfrmMHA_DLL_GAF.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  WhereMouse                  : TPoint;
begin

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;

  if Shift = [ssAlt] then
    begin
      if Key = vk_F4 then
        Close;

      if key in [78, 110] then // New GAF score
        begin
          ActiveControl := spnedtGAF;
          WhereMouse.x := frmMHA_DLL_GAF.Left + spnedtGAF.left +
            spnedtGAF.width;
          WhereMouse.y := frmMHA_DLL_GAF.Top + frmMHA_DLL_GAF.Height - 10;
          Mouse.CursorPos := WhereMouse;
          Key := 0;
        end;
    end;
end;


procedure TfrmMHA_DLL_GAF.grdGAFCriteriaGridHint(Sender: TObject; ARow,
  ACol: Integer; var hintstr: string);
begin
  HintStr := GAFCopyright;
end;

procedure TfrmMHA_DLL_GAF.btnFinishedClick(Sender: TObject);
begin
  SaveTheScore := True;
  if Piece(gvScoreStr, U, 1) = '0' then
    gvScoreStr := '-1';
  Close;
end;

procedure TfrmMHA_DLL_GAF.btnCancelClick(Sender: TObject);
begin
  gvScoreStr := '-1';
  Close;
end;

procedure TfrmMHA_DLL_GAF.spnedtGAFKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = vk_Return then
    btnFinishedClick(Sender);
end;

procedure TfrmMHA_DLL_GAF.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin

  if SaveTheScore then
    CanClose := True
  else
    gvScoreStr := '-1';

  if (gvScoreStr = '-1') then
    begin
      if (MsgDlgFrm('Are you sure you want to exit without saving a GAF score?',
        MHA_DLL_Caption, mtConfirmation, [mbYes, mbNo], 0) =
        mrYes) then
        begin
          gvScoreStr := '-1';
          CanClose := True;
        end
      else
        begin
          CanClose := False;
          gvScoreStr := IntToStr(Round(spnedtGAF.Value)); // + U + DateToStr(dtpkrEval.date);
          SaveTheScore := False;
        end;
    end;
end;

procedure TfrmMHA_DLL_GAF.grdGAFResultsGetCellColor(Sender: TObject; ARow,
  ACol: Integer; AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
begin

  if RowTag(ARow) = 'ERROR' then
    AFont.Style := [fsStrikeOut];
end;

procedure TfrmMHA_DLL_GAF.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  gvTop := Top;
  gvLeft := Left;
  gvHeight := Height;
  gvWidth := Width;

  MHA_DLL_Registry.SaveRegistry;

  //gvFont.Free;
  Action := caFree;
end;

end.

