unit uMHA_DLL_Registry;
{$WARN UNSAFE_CODE OFF}
interface

uses
  Registry,
  SysUtils,
  Windows,
  dialogs,
  WinProcs,
  WinTypes,
  graphics;

type

  TMHA_DLL_Registry = class(TObject)
  private
    function CheckForFilesAndValues: Boolean;
    procedure SetFont(Value: TFont);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure SaveRegistry;
    function ReadRegistry: boolean;
    procedure UseDefaultRegistry;
  end;

var
  MHA_DLL_Registry            : TMHA_DLL_Registry;

implementation

uses
  uIA_Const,
  uMHA_DLL_Const;

const
  WindowsRegistry             : string = 'SOFTWARE\VISTA\YS\MHA_DLL\';

constructor TMHA_DLL_Registry.Create;
begin
  inherited Create;
end;

destructor TMHA_DLL_Registry.Destroy;
begin
  inherited Destroy;
end;

function TMHA_DLL_Registry.ReadRegistry: boolean;
var
  ReadReg                     : TRegistry;
  ReadFont                    : TLogFont;
begin
  Result := False;
  ReadReg := TRegistry.Create;
  try
    with ReadReg do
      begin
        // RootKey := HKEY_LOCAL_MACHINE; Not used
        RootKey := HKEY_CURRENT_USER; // better for Thin client applications
        if OpenKey(WindowsRegistry, true) then
          begin
            if not ValueExists('FirstUse') then
              begin
                CloseKey;
                Free;
                UseDefaultRegistry;
                SaveRegistry;
                Result := true;
                exit;
              end;

            // location of Main form on user's screen
            gvLeft := ReadInteger('UserLeft');
            gvTop := ReadInteger('UserTop');
            gvHeight := ReadInteger('UserHeight');
            gvWidth := ReadInteger('UserWidth');
            gvSpeedTab := ReadBool('SpeedTab');
            gvFirstUse := ReadBool('FirstUse');
            gvHighlightForeground := ReadInteger('UserColorForeground');
            gvHighlightBackground := ReadInteger('UserColorBackground');
            ReadBinaryData('UserFont', ReadFont, SizeOf(ReadFont));
            gvFont.Handle := CreateFontIndirect(ReadFont);
          end;
        CloseKey;
        Free;
      end; {- with ReadReg -}
  except
    on E: exception do
      begin
        // Load default values so program will load
        UseDefaultRegistry;
      end;
  end; {- try -}

  if not CheckForFilesAndValues then
    exit;
  result := true;
end;

procedure TMHA_DLL_Registry.SaveRegistry;
var
  SaveReg                     : TRegistry;
  SaveFont                    : TLogFont;
begin
  //ShowMessage('save '+ gvPatientDFN + #13 + gvInstrumentName);
  SaveReg := TRegistry.Create;
  try
    with SaveReg do
      begin
        RootKey := HKEY_CURRENT_USER; // better for Thin client applications
        if OpenKey(WindowsRegistry, true) then
          begin
            WriteInteger('UserLeft', gvLeft);
            WriteInteger('UserTop', gvTop);
            WriteInteger('UserHeight', gvHeight);
            WriteInteger('UserWidth', gvWidth);
            WriteBool('SpeedTab', gvSpeedTab);
            WriteBool('FirstUse', gvFirstUse);
            WriteInteger('UserColorForeground', gvHighlightForeground);
            WriteInteger('UserColorBackground', gvHighlightBackground);
            GetObject(gvFont.Handle, SizeOf(SaveFont), @SaveFont);
            WriteBinaryData('UserFont', SaveFont, SizeOf(SaveFont));

          end;
        CloseKey;
        Free;
      end; {- with SaveReg -}
  except
    on E: Exception do
      begin
        exit;
      end;
  end; {- try-}
end;

procedure TMHA_DLL_Registry.UseDefaultRegistry;
var
  NonClientMetrics            : TNonClientMetrics;
begin
  //ShowMessage('Getting default values in registry.');
  // Defaults set to zero, see uMHA_DLL for use of this.
  gvTop := 10;
  gvLeft := 10;
  gvWidth := 600;
  gvHeight := 400;

  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    gvFont.Handle := CreateFontIndirect(NonClientMetrics.lfMessageFont);
  SetFont(gvFont);
  gvSpeedTab := True;
  gvFirstUse := True;
  gvHighlightForeground := clBlack;
  gvHighlightBackground := clYellow;

end;

function TMHA_DLL_Registry.CheckForFilesAndValues: boolean;
// This will ensure that all the proper files are present.
begin

  if gvHighlightForeground = gvHighlightBackground then
    begin
      gvHighlightForeground := clBlack;
      gvHighlightBackground := clYellow;
    end;

  SetFont(gvFont);

  Result := true;
end;

procedure TMHA_DLL_Registry.SetFont(Value: TFont);
begin
  gvFont.Assign(Value);
end;

initialization
  MHA_DLL_Registry := TMHA_DLL_Registry.Create;


finalization
  MHA_DLL_Registry.Free;

{$WARN UNSAFE_CODE ON}
end.

