unit uMHA_AUX_ADL;

interface

uses
  SysUtils;

function ScoreAnswers_ADL(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_ADL(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The Katz ADL has very long strings for choice items which requires this system
  to score the items (rather than the usual method within IAT).

  NOT Used: The Index of ADL is a single scale with the result begining a letter from A to G.
  There is no adjustment of blank-skipped answers; all questions must be answered.

*}

function ScoreAnswers_ADL(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    Total                     : Integer;
  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'KATZ ADL' then
  begin
     Result := 'YES';
     exit;
  end;

  Result := '';
  Total := 0;
  Count := NumberOfPieces(AnswerString, RECORD_END);


  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      if TempAnswer <> '0' then
        inc(Total);

    end; // for i

  // Note: "Index" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'Index^^' + inttostr(Total) + RECORD_END;
  
end;


function ProgressNote_ADL(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  TempIndex : string;

begin
  Result := '';
  TempIndex := '';

  if (ScoreString = '') then
    Exit;

  TempIndex := Piece(ScoreString, U, 3); // 6~
  
  Delete(TempIndex, 2, 1);

//  Result := #13 + #13 +
//    Indent + '------------ Katz Independence Index of ADL ----------' + #13;

  Result := Result + Indent +
    'Index: ' + TempIndex + '  (Range 0 to 6)' + #13 + #13 +
    'A high index indicates independence, a low index means dependence on ' + #13 +
    'others to help with activities of daily living.';

  Result := Result + #13;
//    Indent + '------------ Katz Independence Index of ADL Responses ----------' + #13 + #13;

end; 



(*
function ScoreAnswers_ADL(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    Total,
    Bath,
    Dress,
    Toilet,
    Continence,
    Transfer,
    Feed                      : Integer;


  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  Bath := 0;
  Dress := 0;
  Toilet := 0;
  Continence := 0;
  Transfer := 0;
  Feed := 0;

  if Count < 1 then
    Exit;

  for i := 2 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);
      // Bathing
      if (i = 2) then
        if TempAnswer <> '0' then
          Inc(Bath);
      // Dressing
      if (i = 3) then
        if TempAnswer <> '0' then
          Inc(Dress);
      // Toileting
      if (i = 4) then
        if TempAnswer <> '0' then
          Inc(Toilet);
      // Transfer
      if (i = 5) then
        if TempAnswer <> '0' then
          Inc(Transfer);
      // Continence
      if (i = 6) then
        if TempAnswer <> '0' then
          Inc(Continence);
      // Feeding
      if (i = 7) then
        if TempAnswer <> '0' then
          Inc(Feed);
    end; // for i

  Total := Bath + Dress + Toilet + Continence + Transfer + Feed;

  Result := 'Other';

  if (Total = 0) then
    Result := 'A';

  if (Total = 1) then
    Result := 'B';

  if (Total = 2) and (Bath = 1) then
    result := 'C';

  if (Total = 3) and (Bath = 1) and (Dress = 1) then
    result := 'D';

  if (Total = 4) and (Bath = 1) and (Dress = 1) and (Toilet = 1) then
    result := 'E';

  if (Total = 5) and (Bath = 1) and (Dress = 1) and (Toilet = 1) and
    (Transfer = 1) then
    result := 'F';

  if Total = 6 then
    Result := 'G';

end;
*)


(*
function ProgressNote_ADL(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';

begin
  Result := '';

  if (ScoreString = '') then
    Exit;

  Result := #13 + #13 +
    Indent + '------------ Index of ADL ----------' + #13;

  if ScoreString = 'A' then
    Result := Result + #13 + Indent +
      'A - Independent in feeding, continence, transferring, going to toilet,' + #13 +
      Indent +
      '      dressing and bathing.';

  if ScoreString = 'B' then
    Result := Result + #13 + Indent +
      'B - Independent in all but one of these functions; feeding, continence,' + #13 +
      Indent +
      '      transferring, going to toilet, dressing or bathing.';

  if ScoreString = 'C' then
    Result := Result + #13 + Indent +
      'C - Independent in all but bathing and one of these functions;' + #13 +
      Indent +
      '      feeding, continence, transferring, going to toilet, or dressing.';

  if ScoreString = 'D' then
    Result := Result + #13 + Indent +
      'D - Independent in all but bathing, dressing and one of these functions;' + #13 +
      Indent +
      '      feeding, continence, transferring, or going to toilet.';

  if ScoreString = 'E' then
    Result := Result + #13 + Indent +
      'E - Independent in all but bathing, dressing, going to toilet and' + #13 +
      Indent +
      '      one of these functions; feeding, continence, or transferring.';

  if ScoreString = 'F' then
    Result := Result + #13 + Indent +
      'F - Independent in all but bathing, dressing, going to toilet, transferring and' + #13 +
      Indent +
      '      either feeding, or continence.';

  if ScoreString = 'G' then
    Result := Result + #13 + Indent +
      'G - Dependent in all of these functions: bathing, dressing, going to toilet,' + #13 +
      Indent +
      '      transferring, feeding and continence.';

  if ScoreString = 'Other' then
    Result := Result + #13 + Indent +
      'Other - Dependent in at least two of these functions (and not calssified as ' + #13 +
      Indent +
      '      A, B, C, D, E, F, G): bathing, dressing, going to toilet,transferring, ' + #13 +
      Indent +
      '      feeding and continence.';

  Result := Result + #13 + #13 +
    Indent + '------------ Index of ADL Responses ----------' + #13 + #13;



end; *)


end.

