unit uMHA_AUX_DBAS;

interface

uses
  SysUtils;

function ScoreAnswers_DBAS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_DBAS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  DBAS has 16 questions, all track-bar from 0 to 10. Score is average of the
  questions. Note that track-bar items cannot be skipped, so a zero is the
  score for a skipped item here.

*}

function ScoreAnswers_DBAS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var

  Count, i,
    TempAnswer,
    Tally
                              : Integer;

  TempResult
                              : string;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  
  if AnswerString = 'DBAS' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Tally := 0;
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      // Note: Track-bar items cannot be skipped.
      // if TempAnswer <> 1155 then // Question skipped

      Tally := Tally + TempAnswer; // item index atarts at zero
    end; // for i

  // Note: "DBAS" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~

  Result := 'DBAS^^' + FloatToStrF(Tally/16, ffNumber, 1, 1) + RECORD_END;
  //Result := 'DBAS^^' + IntToStr(Tally div 16) + RECORD_END;
end;


function ProgressNote_DBAS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := 'DBAS Score: ' + Piece(Piece(ScoreString, RECORD_END, 1), U, 3);

end;

end.

