unit uMHA_AUX_SBR_II;


interface

uses
  SysUtils,
  Classes,
  Trpcb,
  ORfn;

function ScoreAnswers_SBR_II(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_SBR_II(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


function GetVistaText_SBR_II(
  Broker: TRPCBroker;
  InstrumentName,
  PatientIEN,
  OrderedByDUZ: string): string;


implementation


uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


function ScoreAnswers_SBR_II(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Count, i, Intent,
    TempAnswer                : Integer;
    TempQID,
  TempResult                  : string;
  Thoughts,
    ThoughtsSuicidal,
    Injury,
    Preparatory,
    Interrupted,
    Fatal,
    Unknown,
    NoSDV                     : boolean;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'SBR II' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  NoSDV := false;
  Thoughts := false;
  ThoughtsSuicidal := false;
  Injury := false;
  Intent := 0;
  Preparatory := false;
  Interrupted := false;
  Fatal := false;

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);
      TempQID := Piece(TempResult, EQ, 1);

      if (TempQID = '6697') then // 1) Self-directed violent behavior?
        if (TempAnswer = 0) then // No
          NoSDV := true;

      if (TempQID = '6698') then // 2) Self-directed violent thoughts?
        begin
          if (TempAnswer = 0) and NoSDV then // two No's
            begin
              result := 'SDV Classification^^There is insufficient evidence '+
              'to suggest self-directed violence.'+ RECORD_END;
              exit;
            end;

          if (TempAnswer = 1) then // Yes
            Thoughts := true;
        end;

      if (TempQID = '6699') then // 2A. Thoughts are suicidal?
        if (TempAnswer = 1) then // Yes
          ThoughtsSuicidal := true;

      if (TempQID = '6700') then // 3. Injury?
        if (TempAnswer = 1) then // Yes
          Injury := true;

      if (TempQID = '6701') then // 3A. Injury fatal?
        if (TempAnswer = 1) then // Yes
          Fatal := true;

      if (TempQID = '6702') then // 3B. Preparatory?
        if (TempAnswer = 1) then // Yes
          Preparatory := true;

      if (TempQID = '6703') then // 3C. Behavior interrupted?
        if (TempAnswer = 1) then // Yes
          Interrupted := true;

      if (TempQID = '6704') then // 4. Suicide Intent?
        begin
          // Intent = 0 by default, No
          if (TempAnswer = 1) then // Yes
            Intent := 1;
          if (TempAnswer = 2) then // Unknown
            Intent := 2;
        end;
    end; // for i

   // Decision Tree A: Thoughts
   // Thoughts = No (Default result)
  if Thoughts then
    begin
      result := 'Non-Suicidal SDV Ideation.';
      if ThoughtsSuicidal and (Intent = 0) then // Yes-No
        result := 'Suicidal Ideation, Without Suicidal Intent.';
      if ThoughtsSuicidal and (Intent = 1) then // Yes-Yes
        result := 'Suicidal Ideation, With Suicidal Intent.';
      if ThoughtsSuicidal and (Intent = 2) then // Yes-Unknown
        result := 'Suicidal Ideation, With Undetermined Suicidal Intent';
      result := 'SDV Classification^^' + result + RECORD_END;
      exit;
    end;

   // Decision Tree B: Behaviors without Injury
  if not Injury then
    begin
      if Preparatory and (Intent = 0) then // No-Yes-No
        result := 'Non-Suicidal SDV, Preparatory.';
      if Preparatory and (Intent = 1) then // No-Yes-Yes
        result := 'Suicidal SDV, Preparatory.';
      if Preparatory and (Intent = 2) then // No-Yes-Unknown
        result := 'Undetermined SDV, Preparatory.';

      if not Preparatory then
        begin
          if Interrupted and (Intent = 0) then // No-No-Yes-No
            result := 'Non-Suicidal SDV, Without Injury, Interrupted by Self/Other.';
          if Interrupted and (Intent = 1) then // No-No-Yes-Yes
            result := 'Suicide Attempt, Without Injury, Interrupted by Self/Other.';
          if Interrupted and (Intent = 2) then // No-No-Yes-Unknown
            result := 'Undetermined SDV, Without Injury, Interrupted by Self/Other.';
          if not Interrupted and (Intent = 0) then // No-No-No-No
            result := 'Non-Suicidal SDV, Without Injury.';
          if not Interrupted and (Intent = 1) then // No-No-No-Yes
            result := 'Suicide Attempt, Without Injury.';
          if not Interrupted and (Intent = 2) then // No-No-No-Unknown
            result := 'Undetermined SDV, Without Injury.';
        end; // not Preparatory
    end; // not Injury

  //DECISION TREE C: BEHAVIORS, WITH INJURY
  if Injury then
    begin
      if Fatal and (Intent = 0) then // Yes-Yes-No
        result := 'Non-Suicidal SDV, Fatal.';
      if Fatal and (Intent = 1) then // Yes-Yes-Yes
        result := 'Suicide.';
      if Fatal and (Intent = 2) then // Yes-Yes-Unknown
        result := 'Undetermined SDV, Fatal.';

      if not Fatal then
        begin
          if Interrupted and (Intent = 0) then // Yes-No-Yes-No
            result := 'Non-Suicidal SDV, With Injury, Interrupted by Self/Other.';
          if Interrupted and (Intent = 1) then // Yes-No-Yes-Yes
            result := 'Suicide Attempt, With Injury, Interrupted by Self/Other.';
          if Interrupted and (Intent = 2) then // Yes-No-Yes-Unknown
            result := 'Undetermined SDV, With Injury, Interrupted by Self/Other.';
          if not Interrupted and (Intent = 0) then // Yes-No-No-No
            result := 'Non-Suicidal SDV, With Injury.';
          if not Interrupted and (Intent = 1) then // Yes-No-No-Yes
            result := 'Suicide Attempt, With Injury.';
          if not Interrupted and (Intent = 2) then // Yes-No-No-Unknown
            result := 'Undetermined SDV, With Injury.';
        end; // Fatal
    end; // with injury

  result := 'SDV Classification^^' + result + RECORD_END;
end;


function ProgressNote_SBR_II(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result :=  'SDV Classification: ' +
    Piece(Piece(ScoreString, RECORD_END, 1), U, 3);

end;


function GetVistaText_SBR_II(
  Broker: TRPCBroker;
  InstrumentName,
  PatientIEN,
  OrderedByDUZ: string): string;

// Result is string with

//    Pain Score,            NOT USED
//    Past 10 visits
//    Primary Care Provider  NOT USED
//    Case Manager           NOT USED
//    Provider Rx            NOT USED
//    Active Problem List

//      '5446=11. Last Pain Score: 09/08/11@15:07  7  PROVIDER,CHERYL' + U +
//      '19=15. Past 10 Clinic Visits: ' + #13 + #13 +
//      '   09/10/11 GROUP CLINIC ACTION REQUIRED' + U +
//      '21=17. Primary Care Provider: Provider, Cheryl' + U +
//      '18. Case Manager/Therapist: Therapist, Sam' + U +
//      '19. Name of Provider Prescribing Psychiatric Medications: Med, Giver' + U +
//      '20. Active Problem List: ' + #13 + #13 +
//      '   06/01/11  DOCTOR, ONE  Diabetes ' + #13 +
//      '   09/01/11  PROVIDER,CHERYL Isolation ' + #13 +
//      '   09/01/11  PROVIDER,CHERYL Alcohol abuse ' + #13 +
//      '   09/01/11  PROVIDER,CHERYL Depression ' + U;
const
  DatePicture                 = 'mm/dd/yyyy hh:mm';
var
  Visits,
    i, j                      : integer;
  TempStr,
    TempStr1                  : string;
begin

  with Broker do
    begin
      if PatientIEN = '' then
        exit;
      TempStr := '';
// -----------  Pain Score  ----------------------------------
//   Temp.:       (09/08/05@15:07)  98.9 F  (37.2 C) (ORAL)  _PROVIDER,CHERYL CJL
//   Pulse:       (09/08/05@15:07)  80 (LEFT,BRACHIAL,PALPATED,SITTING)  _PROVIDER,CHERYL CJL
//   Resp.:       (09/08/05@15:07)  14 (SPONTANEOUS,SITTING)  _PROVIDER,CHERYL CJL
//   Pulse Ox:    (09/08/05@15:07)  90%   _PROVIDER,CHERYL CJL
//   B/P:         (09/08/05@15:07)  130/90 (L ARM,SITTING,CUFF,ADULT)  _PROVIDER,CHERYL CJL
//   Ht.:         (09/08/05@15:07)  6 ft  (182.88 cm) (ACTUAL)  _PROVIDER,CHERYL CJL
//   Wt.:         (09/08/05@15:07)  180 lb  (81.65 kg) (ACTUAL,STANDING)  _NABER,DAVID A
//   Body Mass Index:             24.46
//   CVP:         (09/08/05@15:07)  25 cmH2O (18.4 mmHg)*  _PROVIDER,CHERYL CJL
//   Circ/Girth:  (09/08/05@15:07)  30 in (76.20 cm) (ABDOMINAL)  _PROVIDER,CHERYL CJL
//   Pain:        (09/08/05@15:07)  7  _PROVIDER,CHERYL CJL

      Result := '6715='; // QID
      Result := Result + 'Enter pain score just prior to the event.' + U;


// Not used. The pain score should be the one just prior the event.
//      ClearResults := True;
//      ClearParameters := True;
//      CreateContext('OR CPRS GUI CHART');
//      RemoteProcedure := 'GMV LATEST VM';
//      Param[0].Value := PatientIEN;
//      Param[0].PType := Literal;
//
//      Call;
//
//      if Results.Count > 0 then
//        begin
//          for i := 0 to Results.Count - 1 do
//            begin
//              if POS('Pain:', Results[i]) > 0 then
//                begin
//                  TempStr := Results[i];
//                  if Length(TempStr) > 12 then
//                    begin
//                      for j := 12 to Length(TempStr) do
//                        if {(TempStr[j] = '(') or (TempStr[j] = ')') or}
//                          (TempStr[j] = '_') then
//                        else
//                          Result := Result + TempStr[j];
//                      Result := Result + U;
//                    end;
//                end
//            end;
//        end;
//
//      if TempStr = '' then
//        Result := Result + 'No pain score found.' + U;


// ------------- Past 10 Visits  --------------------------------------
// Lists appts/visits for a patient (uses same call as cover sheet)
//  V|A;DateTime;LocIEN^DateTime^LocName^Status
//  V;3090330.10515;76^3090330.10515^GROUP CLINIC^ACTION REQUIRED

      TempStr := '';
      TempStr1 := '';
      Visits := 0;

      ClearResults := True;
      ClearParameters := True;
      CreateContext('OR CPRS GUI CHART');

      RemoteProcedure := 'ORWCV VST';
      Param[0].Value := PatientIEN;
      //Param[4].Value := NOW;
      Param[0].PType := Literal;

      Call;

      Result := Result + '6721='; // QID

      if Results.Count > 0 then
        begin
          SortByPiece(TStringList(Results), U, 2);
          InvertStringList(TStringList(Results));
          SetListFMDateTime(DatePicture, TStringList(Results), U, 2);

          for i := 0 to Results.Count - 1 do
            begin
              TempStr := Results[i];

              // No future visits
              if StrToDateDef(Piece(Piece(TempStr, U, 2), ' ', 1), Now) < Now  then
                begin
                  TempStr1 := TempStr1 + '   ' +
                 {Date Modified}Piece(Piece(TempStr, U, 2), ' ', 1) + '  ' +
                 {Appt}MixedCase(Piece(TempStr, U, 3)) + '  ' +
                 {Scheduled}Piece(TempStr, U, 4) + RECORD_END;

                  Inc(Visits);
                  if Visits > 9 then
                    break;
               end;
            end;
        end;

      if TempStr1 <> '' then
        Result := Result + TempStr1 + U
      else
        Result := Result + 'No visits found.' + U;


// ------------- Primary Care Provider  ------------------------------
// Not used: The returned provider may, or may not, be the primary provider.
//
//
// Results look like:
//   [DATA]             4558^ACKERMAN,NIEN-CHIN
//   [ERROR]            0^PC PROVIDER NOT FOUND FOR THIS OUTPATIENT
//   [ERROR]            0^PC PROVIDER NOT FOUND FOR THIS INPATIENT
//   [DATA]             108^JURASAS,ERIC E

//** Not used: User to supply the name of primary.

//Result := Result + '5448=' + U; // QID


//      ClearResults := True;
//      ClearParameters := True;
//      CreateContext('YS BROKER1');
//      RemoteProcedure := 'YTQ PC PROVIDER';
//      Param[0].Mult['"DFN"'] := PatientIEN;
//      Param[0].PType := List;
//
//      Call;
//
//      Result := Result + '5448='; // QID
//
//      if Results[0] = DATA then
//        Result := Result + Piece(results[1], U, 2) + U
//      else
//        Result := Result + 'Primary provider not found.' + U;


//------------- Active Problem List  ---------------------------------
//10^A^Alcohol Dependence (ICD-9-CM 303.90/303.91)^303.90^^3060125^^^P^17;CARDIOLOGY^C^6620;FINKELSTEIN,ALLAN^^^0^^^
//1^A^STRIKES OTHER PATIENTS^799.9^2950804^2950918^NSC^^P^36;RED CLINIC^C^178;DEVLIN,VITALIA^^C^1^2950918^NSC^
//2^A^THROWS FOOD^799.9^2950820^2950918^NSC^^P^36;RED CLINIC^C^178;DEVLIN,VITALIA^^C^1^2950918^NSC^
//618^A^Carcinoma of Head and Neck^195.0^3011201^03/14/2002 00:03^NSC^^P^7;1 CARY'S CLINIC^C^18;TROST,DEBBIE^^^0^3020314^NSC^1 1^^^^^^
      TempStr := '';
      TempStr1 := '';
      ClearResults := True;
      ClearParameters := True;
      CreateContext('OR CPRS GUI CHART');
      RemoteProcedure := 'ORQQPL PROBLEM LIST';
      Param[0].Value := PatientIEN;
      Param[0].PType := Literal;

      Call;

      if Results.Count > 0 then
        begin
          SortByPiece(TStringList(Results), U, 6);
          InvertStringList(TStringList(Results));
          SetListFMDateTime(DatePicture, TStringList(Results), U, 6);

          Result := Result + '6727='; //QID
          TempStr1 := '';

          for i := 0 to Results.Count - 1 do
            begin
              TempStr := Results[i];

              if POS('No data available', TempStr) > 0 then
                TempStr1 := 'No data available.'
              else
                begin
                  if (TempStr <> '') then
                    if (Piece(TempStr, U, 2) = 'A') then
                      TempStr1 := TempStr1 + '   ' +
                      {Date Modified}Piece(Piece(TempStr, U, 6), ' ', 1) + '  ' +
                // {Date Modified}Piece(TempStr, U, 6) + '  ' +
                 {Provider}Piece(Piece(TempStr, U, 12), ';', 2) + '  ' +
                 {Problem}MixedCase(Piece(TempStr, U, 3)) + RECORD_END;
                end;
            end;
          Result := Result + TempStr1 + U;
        end;
    end; // with Broker

end;



end.

