unit uMHA_AUX_WHOQOL_BREF;

interface

uses
  SysUtils;

function ScoreAnswers_WHOQOL_BREF(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_WHOQOL_BREF(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The WHOQOL-BREF has four domain scores, each 0-100.

   Physical domain= ((6-Q3) + (6-Q4) + Q10 + Q15 + Q16 + Q17 + Q18)x4.
   Psychological domain= (Q5 + Q6 + Q7 + Q11 + Q19 + (6-Q26))x4.
   Social Relationships domain= (Q20 + Q21 + Q22)x4.
   Environment domain= (Q8 + Q9 + Q12 + Q13 + Q14 + Q23 + Q24 + Q25)x4.

*}


// Scoring method is provided in WHODAS manual. There are two mehtods: simple
// and complex. We use the complex; it is more accurate.

function ScoreAnswers_WHOQOL_BREF(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    Skip,
    PhyM, PsychM, RelM, EnvM, // Missing items
    Phy, Psych, Rel, Env      : Integer;


  PhyStr, PsychStr, RelStr, EnvStr,
    TempResult,
    TempStr1, TempStr2,
    TempAnswer                : string;

  // patch 108 ver 70, need a temporary var for reverse scoring, bug fix
  revTemp :                   integer ;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'WHOQOL-BREF' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;
  Skip := 0;
  Phy := 0;
  Psych := 0;
  Rel := 0;
  Env := 0;

  PhyM := 0;
  PsychM := 0;
  RelM := 0;
  EnvM := 0;

  PhyStr := '';
  PsychStr := '';
  RelStr := '';
  EnvStr := '';

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Count the number of skipped (Blank) answers for each scale
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        //(TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
      (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          TempAnswer := '0';
          Inc(Skip);

          if (i = 3) or (i = 4) or (i = 10) or (i = 15) or (i = 16) or
            (i = 17) or (i = 18) then
            Inc(PhyM);

          if (i = 5) or (i = 6) or (i = 7) or (i = 11) or (i = 19) or
            (i = 26) then
            Inc(PsychM);

          if (i = 20) or (i = 21) or (i = 22) then
            Inc(RelM);

          if (i = 8) or (i = 9) or (i = 12) or (i = 13) or (i = 14) or
            (i = 23) or (i = 24) or (i = 25) then
            Inc(EnvM);

        end // Skipped questions
      else
        begin
          // patch 108, v70 llh, added logic for reverse scoring these 3 questions
          // item index is used rather than the value, therefore must do 2 conversions
          // itemindex = 1 less than score, use itemindex, then adjust score below
          if ( (i = 3) or  (i = 4) or  (i = 26) ) then
          begin
            revTemp := StrToInt(tempAnswer) ;
            case revTemp of
              0: revTemp := 4 ;
              1: revTemp := 3 ;
              2: revTemp := 2 ;
              3: revTemp := 1 ;
              4: revTemp := 0 ;
             else
               revTemp := 0 ;
             end;
             tempAnswer := Inttostr(revTemp) ;
           end ;
          // end added reverse scoring logic, llh


          // Physical
          // reverse score calculated above, commented 2 lines, added i=3 and i=4 below
          //if (i = 3) or (i = 4) then // Reverse answers
            //Phy := Phy + (5 - StrToIntDef(TempAnswer, 0));

          if (i = 3) or (i = 4) or (i = 10) or (i = 15) or (i = 16) or (i = 17) or (i = 18) then
            Phy := Phy + StrToIntDef(TempAnswer, 0) + 1;

          // Psychological
          // reverse score calculated above, this line was incorrect pre ver 70, no change needed
          // (question 26 was not reverse scored originally)
          if (i = 5) or (i = 6) or (i = 7) or (i = 11) or (i = 19) or (i = 26) then
            Psych := Psych + StrToIntDef(TempAnswer, 0) + 1;

          // Social Relationships
          if (i = 20) or (i = 21) or (i = 22) then
            Rel := Rel + StrToIntDef(TempAnswer, 0) + 1;
                               
          // Environment
          if (i = 8) or (i = 9) or (i = 12) or (i = 13) or (i = 14) or
            (i = 23) or (i = 24) or (i = 25) then
            Env := Env + StrToIntDef(TempAnswer, 0) + 1;

        end; // for i
    end;

  // if more than two items are skipped, then bail out.
  // patch 108 ver 70, changed skip > 4 to skip > 5, if num questions answered >= 21 score,
  //                   otherwise don't
  if Skip > 5 then
    begin
      Result := 'Physical^^X' + RECORD_END +
        'Psychological^^X' + RECORD_END +
        'Social Relationships^^X' + RECORD_END +
        'Environment^^X' + RECORD_END;
      exit;
    end;

  // If missing Physical score: Use the average of the other scores for missing
  // score.
  if (PhyM > 0) then
    Phy := (PhyM * (Phy div (7 - PhyM))) + Phy;

  // Missing Psych score
  if (PsychM > 0) then
    Psych := (PsychM * (Psych div (6 - PsychM))) + Psych;

  // Missing Social Relationships score
  if (RelM > 0) then
    Rel := (RelM * (Rel div (3 - RelM))) + Rel;

  // Missing Environment score
  if (EnvM > 0) then
    Env := (EnvM * (Env div (8 - EnvM))) + Env;


  // Too many missing scores
  if (PhyM > 1) then
    PhyStr := 'Too many missing responses to score.';

 // Too many Missing Psych score
  if (PsychM > 1) then
    PsychStr := 'Too many missing responses to score.';

  // Too many Missing Social Relationships score
  // patch 108 ver 70, social relationships can have 1 skipped question, changed from 0 to 1
  if (RelM > 1) then
    RelStr := 'Too many missing responses to score.';

  // Too many Missing Environment score
  if (EnvM > 2) then
    EnvStr := 'Too many missing responses to score.';


  // Score is mean multiplied by 4, then set as T-score (range 0-100).
  if PhyStr = '' then
    PhyStr := FloatToStrF((((Phy / 7) * 4) - 4) * (100 / 16), ffNumber, 12, 2);

  if PsychStr = '' then
    PsychStr := FloatToStrF((((Psych / 6) * 4) - 4) * (100 / 16), ffNumber, 12, 2);

  if RelStr = '' then
    RelStr := FloatToStrF((((Rel / 3) * 4) - 4) * (100 / 16), ffNumber, 12, 2);

  if EnvStr = '' then
    EnvStr := FloatToStrF((((Env / 8) * 4) - 4) * (100 / 16), ffNumber, 12, 2);

  Result := 'Physical^^' + PhyStr + RECORD_END +
    'Psychological^^' + PsychStr + RECORD_END +
    'Social Relationships^^' + RelStr + RECORD_END +
    'Environment^^' + EnvStr + RECORD_END;
end;


function ProgressNote_WHOQOL_BREF(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer, questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := #13 + #13 + ' WHO Quality of Life Domains' + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3);
      if TempStr2 = 'X' then
        begin
          Result := 'Too many missing answers. Max is 5.';
          exit;
        end;
      Result := Result + Indent + TempStr1 + ': ' + TempStr2 + #13;
    end;

  Result := Result + #13 + Indent +
    'Range is 0 to 100 where a high score indicates increase level of quality.';
end;

end.

