unit uIA_Question_IO;

(*

1) The relationship between Question and Introduction is a congruence between their
respective ObjectLists. Introduction[1] is for Question[1], and so on.
The ObjectList starts at zero. The Introduction appears on the form before the
Question it is linked with.

*)

interface

uses
  Trpcb,
  classes,
  System.Contnrs,
  System.UITypes,
  uIA_Instrument;

function ReadQuestionAndIntroLists(Broker: TRPCBroker; var aInstrument: TInstrument): string;

implementation

uses
  SysUtils,
  fIA_dialogs,
  graphics,
  uIA_Const,
  uIA_Utils,
  uIA_Question,
  uIA_Question_OPS,
  uIA_Introduction,
  uIA_Display,
  uIA_Display_OPS;


function ReadQuestionAndIntroLists(Broker: TRPCBroker; var aInstrument: TInstrument): string;
var
  TempStr,
    LastIntro                 : string;
  i, x                        : integer;
  AQuestion                   : TQuestion;
  AIntroduction               : TIntroduction;
  TempList                    : TStringList;

  procedure VerifyIntroduction;
  begin
    if AIntroduction.Display.DisplayFont.Name = '' then
      AIntroduction.Display.DisplayFont.Name := 'MS Sans Serif';

    if (AIntroduction.Display.DisplayFont.Size > 18) or
      (AIntroduction.Display.DisplayFont.Size < 8) then
      AIntroduction.Display.DisplayFont.Size := 10;

    if (AIntroduction.Display.DisplayLeft > 500) or
      (AIntroduction.Display.DisplayLeft < 0) then
      AIntroduction.Display.DisplayLeft := 10;
  end;

  procedure VerifyQuestion;
  begin
    if AQuestion.Display.DisplayFont.Name = '' then
      AQuestion.Display.DisplayFont.Name := 'MS Sans Serif';

    if (AQuestion.Display.DisplayFont.Size > 18) or
      (AQuestion.Display.DisplayFont.Size < 8) then
      AQuestion.Display.DisplayFont.Size := 10;

    if (AQuestion.Display.DisplayLeft > 500) or
      (AQuestion.Display.DisplayLeft < 0) then
      AQuestion.Display.DisplayLeft := 10;
  end;

  procedure SetDefault(IsAnswer: Boolean; var ThisDisplay: TDisplay);
  var
    TempFont                  : TFont;
  begin
    TempFont := TFont.Create;
    TempFont.Name := 'MS Sans Serif';
    TempFont.Size := 10;
    TempFont.Color := clWindowText;
    TempFont.Style := [];

    with ThisDisplay do
      begin
        //DisplayID := GetDisplayID(TempFont,
        //  taLeftJustify, 2, False, NA, 0, NA);
        DisplayFont.Name := TempFont.Name;
        DisplayFont.Style := [];
        DisplayFont.Size := TempFont.Size;
        DisplayFont.Color := TempFont.Color;
        DisplayAlignment := taLeftJustify;
        DisplayLeft := 2;

        if IsAnswer then
          begin
            DisplayMask := '';
            DisplayColumns := 1;
            DisplayComponent := acRadioGroup;
            DisplaySameLine := True;
          end;
      end;
    TempFont.Free;
  end; // SetDefault

begin
  Result := OK;
  LastIntro := '';

  with Broker, Display_OPS, Question_OPS do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ QUESTALL';
      Param[0].Mult['"CODE"'] := aInstrument.InstrumentName;
      Param[0].PType := List;
      Call;

      aInstrument.NumberOfQuestions := StrToIntDef(Piece(Results[1], EQ, 2), 0);

      // The case of a brand new instrument
      if aInstrument.NumberOfQuestions = 0 then
        Exit;

//      if (Results.Count - 4) mod 7 <> 0 then
//        begin
//          Result := 'This instrument cannot be displayed. The questions were ' +
//            'saved in a format that cannot be read.';
//          Exit;
//        end;

      TempList := TStringList.Create;

      for i := 2 to Results.Count - 1 do
        TempList.Add(Results[i]);

      for i := 0 to TempList.Count - 1 do
        begin

          if Pos('QUESTION NUMBER', TempList[i]) > 0 then
            begin

              AQuestion := TQuestion.Create;
              TempStr := Piece(TempList[i], EQ, 2);
              AQuestion.QuestionID := Piece(TempStr, U, 1);
              AQuestion.qTag := StrToIntDef(AQuestion.QuestionID, -1);

              if i = 2 then
                AQuestion.IsFirstQuestion := True
              else
                AQuestion.IsFirstQuestion := False;

              AQuestion.InstrumentContentsSequenceField := Piece(TempStr, U, 2);
              AQuestion.Designator := Piece(TempStr, U, 3);
              AQuestion.InstrumentContentsID := Piece(TempStr, U, 4);
            end;

          if Pos('QUESTION TEXT', TempList[i]) > 0 then
            begin

              AQuestion.qText := Piece(TempList[i], EQ, 2);

              // The question text can take several lines
              x := 1;
              while (Pos('INTRO TEXT', TempList[i + x]) = 0) and (x < 10) do
                begin
                  AQuestion.qText := AQuestion.qText + ' ' + TempList[i + x];
                  Inc(x);
                end;

              //Set caption
              if (AQuestion.Designator = '') then
                AQuestion.qCaption := AQuestion.qText
              else
                begin
                  if StrToIntDef(AQuestion.Designator, -1) = -1 then
                    AQuestion.qCaption := AQuestion.Designator + SEPARATOR +
                      AQuestion.qText
                  else
                    AQuestion.qCaption := AQuestion.Designator + '.' +
                      SEPARATOR + AQuestion.qText;
                end;
            end;

          if Pos('INTRO TEXT', TempList[i]) > 0 then
            begin

              TempStr := Piece(TempList[i], EQ, 2);

              AIntroduction := TIntroduction.Create;
              AIntroduction.ID := Piece(TempStr, U, 1);
              AIntroduction.iText := Piece(TempStr, U, 2);

              // Look for more lines of introductory text
              if (i < TempList.Count - 2) then
                begin
                  x := 1;
                  while (Pos('DESC', TempList[i + x]) = 0) and (x < 10) and
                    ((i + x) < TempList.Count) do
                    begin
                      AIntroduction.iText := AIntroduction.iText + ' ' + TempList[i + x];
                      Inc(x);
                    end;
                end;

              //AIntroduction.iText := RemoveCR(AIntroduction.iText);
              AIntroduction.iText :=
                StringReplace(AIntroduction.iText, '  ', ' ', [rfReplaceAll]);

              if AIntroduction.iText = LastIntro then
                AIntroduction.IsNew := false
              else
                AIntroduction.IsNew := true;

              LastIntro := AIntroduction.iText;
              // The Introduction is added to the ListOfIntroductions below, in
              // the IDISPLAY statement
            end;

          if Pos('DESC', TempList[i]) > 0 then
          // DESC1= Response Type^CHOICETYPE^MIN^MAX^IS REQUIRED^HINT^CHOICEIDENTIFIER
            begin

              TempStr := Piece(TempList[i], EQ, 2);

              AQuestion.ResponseType := ConvertStringToResponseType(Piece(TempStr, U, 1));
              AQuestion.ChoiceTypeID := Piece(TempStr, U, 2);
              AQuestion.qMin := StrToIntDef(Piece(TempStr, U, 3), 0);
              AQuestion.qMax := StrToIntDef(Piece(TempStr, U, 4), 0);
              AQuestion.IsRequired := StrToBool(Piece(TempStr, U, 5));
              AQuestion.qHint := Piece(TempStr, U, 6);
              AQuestion.ChoiceIdentifier := ConvertStringToChoiceIdentifier(Piece(TempStr, U, 7));
            end;

          if Pos('QDISPLAY', TempList[i]) > 0 then
          //  DisplayId^FontName^FontBold^FontItalic
          //  ^FontUnderlined ^ Fontsize^FontColor ^Alignment
            begin
              if Pos('ERROR', TempList[i]) > 0 then
                TempStr := ''
              else
                TempStr := Piece(TempList[i], EQ, 2);

              if TempStr = '' then
                SetDefault(False {not answer}, AQuestion.Display)
              else
                begin
                  AQuestion.Display.DisplayID := Piece(TempStr, U, 1);
                  AQuestion.Display.DisplayFont.Name := Piece(TempStr, U, 2);
                  if Piece(TempStr, U, 3) = 'Yes' then
                    AQuestion.Display.DisplayFont.Style := [fsBold];
                  if Piece(TempStr, U, 4) = 'Yes' then
                    AQuestion.Display.DisplayFont.Style :=
                      AQuestion.Display.DisplayFont.Style + [fsItalic];
                  if Piece(TempStr, U, 5) = 'Yes' then
                    AQuestion.Display.DisplayFont.Style :=
                      AQuestion.Display.DisplayFont.Style + [fsUnderline];
                  AQuestion.Display.DisplayFont.Size := StrToIntDef(Piece(TempStr, U, 6), 8);
                  AQuestion.Display.DisplayFont.Color := StringToColor(Piece(TempStr, U, 7));
                  AQuestion.Display.DisplayAlignment :=
                    StrToAlignment(Piece(TempStr, U, 8));
                  AQuestion.Display.DisplayLeft := StrToIntDef(Piece(TempStr, U, 9), 1);
                end;
            end;

          if Pos('IDISPLAY', TempList[i]) > 0 then
          // DisplayId^FontName^FontBold^FontItalic
          // ^FontUnderlined ^Fontsize^FontColor^Alignment
            begin

              if Pos('ERROR', TempList[i]) > 0 then
                TempStr := ''
              else
                TempStr := Piece(TempList[i], EQ, 2);

              if TempStr = '' then
                SetDefault(False {not answer}, AIntroduction.Display)
              else
                begin
                  AIntroduction.Display.DisplayID := Piece(TempStr, U, 1);
                  AIntroduction.Display.DisplayFont.Name := Piece(TempStr, U, 2);
                  if Piece(TempStr, U, 3) = 'Yes' then
                    AIntroduction.Display.DisplayFont.Style := [fsBold];
                  if Piece(TempStr, U, 4) = 'Yes' then
                    AIntroduction.Display.DisplayFont.Style :=
                      AIntroduction.Display.DisplayFont.Style + [fsItalic];
                  if Piece(TempStr, U, 5) = 'Yes' then
                    AIntroduction.Display.DisplayFont.Style :=
                      AIntroduction.Display.DisplayFont.Style + [fsUnderline];
                  AIntroduction.Display.DisplayFont.Size := StrToIntDef(Piece(TempStr, U, 6), 8);
                  AIntroduction.Display.DisplayFont.Color := StringToColor(Piece(TempStr, U, 7));
                  AIntroduction.Display.DisplayAlignment :=
                    StrToAlignment(Piece(TempStr, U, 8));
                  AIntroduction.Display.DisplayLeft := StrToIntDef(Piece(TempStr, U, 9), 1);
                  VerifyIntroduction;
                end;

              aInstrument.ListOfIntroductions.Add(AIntroduction);

            end;

          // This is the last statement for this question object
          if Pos('CDISPLAY', TempList[i]) > 0 then
          //  CDISPLAY1=DisplayId^FontName^FontBold^FontItalic
          //  ^FontUnderlined ^Fontsize^FontColor^Alignment
          //  ^left^Mask^Colums^Components
            begin

              if Pos('ERROR', TempList[i]) > 0 then
                TempStr := ''
              else
                TempStr := Piece(TempList[i], EQ, 2);

              if TempStr = '' then
                SetDefault(True {not answer}, AQuestion.AnswerDisplay)
              else
                begin
                  AQuestion.AnswerDisplay.DisplayID := Piece(TempStr, U, 1);
                  AQuestion.AnswerDisplay.DisplayFont.Name := Piece(TempStr, U, 2);
                  if Piece(TempStr, U, 3) = 'Yes' then
                    AQuestion.AnswerDisplay.DisplayFont.Style := [fsBold];
                  if Piece(TempStr, U, 4) = 'Yes' then
                    AQuestion.AnswerDisplay.DisplayFont.Style :=
                      AQuestion.AnswerDisplay.DisplayFont.Style + [fsItalic];
                  if Piece(TempStr, U, 5) = 'Yes' then
                    AQuestion.AnswerDisplay.DisplayFont.Style :=
                      AQuestion.AnswerDisplay.DisplayFont.Style + [fsUnderline];
                  AQuestion.AnswerDisplay.DisplayFont.Size := StrToIntDef(Piece(TempStr, U, 6), 8);
                  AQuestion.AnswerDisplay.DisplayFont.Color := StringToColor(Piece(TempStr, U, 7));
                  AQuestion.AnswerDisplay.DisplayAlignment :=
                    StrToAlignment(Piece(TempStr, U, 8));
                  AQuestion.AnswerDisplay.DisplayLeft := StrToIntDef(Piece(TempStr, U, 9), 1);
                  AQuestion.AnswerDisplay.DisplayMask := Piece(TempStr, U, 10);
                  AQuestion.AnswerDisplay.DisplayColumns := StrToIntDef(Piece(TempStr, U, 11), 1);
                  AQuestion.AnswerDisplay.DisplayComponent :=
                    StrToAnswerComponent(Piece(TempStr, U, 12));
                  AQuestion.AnswerDisplay.DisplaySameLine :=
                    (Piece(AQuestion.AnswerDisplay.DisplayMask, DELIMITER, 3) = SAME_LINE);
                end;

              if i = (TempList.Count - 1) then
                AQuestion.IsLastQuestion := true
              else
                AQuestion.IsLastQuestion := False;
              // There are seven statements for each AQuestion.
              // Save the question object after the 7th statement
              VerifyQuestion;
              aInstrument.ListOfQuestions.Add(AQuestion);
            end;
        end;

      TempList.Free;
    end; // Broker...
end;


end.

