unit uIA_SectionLabel_IO;

interface

uses
  Trpcb,
  graphics,
  classes,
  System.Contnrs,
  System.UITypes,
  uIA_Instrument;

function ReadSectionList(Broker: TRPCBroker; NeedIEN: Boolean;
  var aInstrument: TInstrument): boolean;

implementation

uses
  SysUtils,
  fIA_dialogs,
  uIA_Const,
  uIA_Utils,
  uIA_Display,
  uIA_Display_OPS,
  uIA_SectionLabel;

function ReadSectionList(Broker: TRPCBroker; NeedIEN: Boolean;
  var aInstrument: TInstrument): boolean;
// Return looks like this:
// 3934^Desc^I. DESCRIPTION AND BACKGROUND ^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3947^Mil^II. MILITARY ^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3952^Hist^III. PSYCHIATRIC/SUBSTANCE USE TREATMENT HISTORY^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3955^Med^IV. MEDICAL HISTORY^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3959^Empl^V. EMPLOYMENT/EDUCATIONAL STATUS^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3966^Legal^VI. LEGAL HISTORY^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3967^Diag^VII. DIAGNOSES AND PROBLEM AREAS NEEDING TREATMENT^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3981^Ref^VIII. REFERRAL^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3982^Suppl^IX. SUPPLEMENTAL INFORMATION^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
var
  TempStr                     : string;
  TempStrList                 : TStringList;
  i, j                        : Integer;
  ASectionLabel               : TSectionlabel;
begin
  Result := true;

  with Broker, Display_OPS do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');

      RemoteProcedure := 'YTQ SECTION';
      Param[0].Mult['"CODE"'] := aInstrument.InstrumentName;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        begin
          Result := false;
          Exit;
        end; //if not results[0]...

      for i := 1 to Results.Count - 1 do
        begin

          // There are two statements for each section.
          // This is the 2nd statement. Format same as Display types:
          // DISPLAY=209^FontName^....
          if Pos('DISPLAY', Results[i]) > 0 then
            begin
              TempStr := Piece(Results[i], EQ, 2);


              ASectionLabel.Display.DisplayID := Piece(TempStr, U, 1);
              ASectionLabel.Display.DisplayFont.Name := Piece(TempStr, U, 2);
              if Piece(TempStr, U, 3) = 'Yes' then
                ASectionLabel.Display.DisplayFont.Style := [fsBold];
              if Piece(TempStr, U, 4) = 'Yes' then
                ASectionLabel.Display.DisplayFont.Style :=
                  ASectionLabel.Display.DisplayFont.Style + [fsItalic];
              if Piece(TempStr, U, 5) = 'Yes' then
                ASectionLabel.Display.DisplayFont.Style :=
                  ASectionLabel.Display.DisplayFont.Style + [fsUnderline];
              ASectionLabel.Display.DisplayFont.Size := StrToIntDef(Piece(TempStr, U, 6), 8);
              ASectionLabel.Display.DisplayFont.Color := StringToColor(Piece(TempStr, U, 7));
              ASectionLabel.Display.DisplayAlignment :=
                StrToAlignment(Piece(TempStr, U, 8));
              ASectionLabel.Display.DisplayLeft := StrToIntDef(Piece(TempStr, U, 9), 1);
              aInstrument.ListOfSections.Add(ASectionLabel);
            end
          else
          // First statement
          // Format:  QuestionID^TabStr^CaptionStr^DisplayID

            begin
              if NumberOfPieces(Results[i], U) > 2 then
              // Occasionally DisplayID is on a second line (extra carriage return in
              // CaptionStr?) Might be caused by spell check.
                begin
                  ASectionLabel := TSectionLabel.Create;
                  ASectionLabel.Clear;

                  ASectionLabel.QuestionID := Piece(Results[i], U, 1);
                  ASectionLabel.TabCaption := Piece(Results[i], U, 2);
                  ASectionLabel.SectionCaption := Piece(Results[i], U, 3);

                end;
            end;
        end; //for i

      if not NeedIEN then
        begin
          // Needed for DLL, in case there is a crash
          CreateContext('OR CPRS GUI CHART');
          Exit;
        end;

      // The above captured everything except the section's IEN.
      if aInstrument.ListOfSections.Count = 0 then
        Exit;

      TempStrList := TStringList.Create;
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      Remoteprocedure := 'YTQ GENERIC LISTER';
      Param[0].Mult['"FILEN"'] := '601.81';
      Param[0].Mult['"FIELD"'] := '.01';
      Param[0].Mult['"FIELD"'] := '2';
      Param[0].Mult['"NUMBER"'] := '5000';
      Param[0].Mult['"FLAG"'] := 'I';
      Param[0].PType := List;
      Call;

      for i := 2 to Results.Count - 1 do
        TempStrList.Add(Results[i]);

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; //with DataModuleEdit.RPCBroker1...

  for i := 0 to TempStrList.Count - 1 do
    begin
      TempStr := Piece(TempStrList[i], U, 2);

      if (TempStr <> '') then
        begin
          for j := 0 to aInstrument.ListOfSections.Count - 1 do
            begin
              if TempStr =
                TSectionLabel(aInstrument.ListOfSections[j]).QuestionID then
                begin
                  TSectionLabel(aInstrument.ListOfSections[j]).IEN :=
                    Piece(TempStrList[i], U, 1);
                end;
            end; // for j
        end;
    end; // for i

  TempStrList.Free;

end;
(*
function ReadSectionList(AInstrumentName: string; Broker: TRPCBroker): boolean;
// Return looks like this:
// 3934^Desc^I. DESCRIPTION AND BACKGROUND ^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3947^Mil^II. MILITARY ^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3952^Hist^III. PSYCHIATRIC/SUBSTANCE USE TREATMENT HISTORY^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3955^Med^IV. MEDICAL HISTORY^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3959^Empl^V. EMPLOYMENT/EDUCATIONAL STATUS^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3966^Legal^VI. LEGAL HISTORY^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3967^Diag^VII. DIAGNOSES AND PROBLEM AREAS NEEDING TREATMENT^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3981^Ref^VIII. REFERRAL^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
// 3982^Suppl^IX. SUPPLEMENTAL INFORMATION^1485
// DISPLAY=1485^MS Sans Serif^Yes^No^No^10^clWindowText^Left^2^^^
var
  TempStr                     : string;
  TempStrList                 : TStringList;
  i, j                        : Integer;
  ASectionLabel               : TSectionlabel;
begin
  Result := true;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');

      RemoteProcedure := 'YTQ SECTION';
      Param[0].Mult['"CODE"'] := AInstrumentName;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        begin
          Result := False;
          Exit;
        end; //if not results[0]...

      for i := 1 to Results.Count - 1 do
        begin
          // There are two statements for each section.
          // This is the 2nd statement. Format same as Display types:
          // DISPLAY=209^FontName^....
          if Pos('DISPLAY', Results[i]) > 0 then
            begin
              TempStr := Piece(Results[i], EQ, 2);
              ASectionLabel.DisplayRecord := Pieces(TempStr, U, 2, 12);
              Instrument.ListOfSections.Add(ASectionLabel);
            end
          else
          // First statement
          // Format:  QuestionID^TabStr^CaptionStr^DisplayID
            begin
              ASectionLabel := TSectionLabel.Create;
              ASectionLabel.QuestionID := Piece(Results[i], U, 1);
              ASectionLabel.TabCaption := Piece(Results[i], U, 2);
              ASectionLabel.SectionCaption := Piece(Results[i], U, 3);
              ASectionLabel.DisplayID := Piece(Results[i], U, 4);
            end;
        end;

      // The above captured everything except the section's IEN.
      if Instrument.ListOfSections.Count = 0 then
        Exit;

      TempStrList := TStringList.Create;
      ClearParameters := True;
      ClearResults := True;

      Remoteprocedure := 'YTQ GENERIC LISTER';
      Param[0].Mult['"FILEN"'] := '601.81';
      Param[0].Mult['"FIELD"'] := '.01';
      Param[0].Mult['"FIELD"'] := '2';
      Param[0].Mult['"NUMBER"'] := '5000';
      Param[0].Mult['"FLAG"'] := 'I';
      Param[0].PType := List;
      Call;

      for i := 1 to Results.Count - 1 do
          TempStrList.Add(Results[i]);

    end; //with Broker

 for i := 1 to TempStrList.Count - 1 do
    begin
      TempStr := Piece(TempStrList[i], U, 2);
      if TempStr <> '' then
        begin
          for j := 0 to Instrument.ListOfSections.Count - 1 do
            begin
              if TempStr =
                TSectionLabel(Instrument.ListOfSections[j]).QuestionID then
                begin
                  TSectionLabel(Instrument.ListOfSections[j]).IEN :=
                    Piece(TempStrList[i], U, 1);
                end;
            end; // for j
        end;
    end; // for i

  TempStrList.Free;
end;   *)

end.

 